@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -23,6 +23,7 @@
 #include <qfont.h>
 #include <qwhatsthis.h>
 #include <qapplication.h>
+#include <qpushbutton.h>
 
 #include <kapplication.h>
 #include <kmessagebox.h>
@@ -97,23 +98,14 @@ void ModuleTitle::clear()
   kapp->processEvents();
 }
 
-class ModuleWidget : public QVBox
-{
-  public:
-    ModuleWidget( QWidget *parent, const char *name );
-    ~ModuleWidget() {}
-
-    ProxyWidget* load( ConfigModule *module );
-
-  protected:
-    ModuleTitle *m_title;
-    QVBox *m_body;
-};
-
 ModuleWidget::ModuleWidget( QWidget *parent, const char *name )
     : QVBox( parent, name )
 {
-  m_title = new ModuleTitle( this, "m_title" );
+  QHBox* titleLine = new QHBox( this, "titleLine");
+  m_title = new ModuleTitle( titleLine, "m_title" );
+  QPushButton *helpButton = new QPushButton( titleLine );
+  helpButton->setIconSet( SmallIconSet("help") );
+  connect (helpButton, SIGNAL( clicked() ), this, SIGNAL( helpRequest() ) );
   m_body = new QVBox( this, "m_body" );
   setStretchFactor( m_body, 10 );
 }
@@ -145,6 +137,7 @@ DockContainer::DockContainer(QWidget *parent)
   addWidget( _busyw );
 
   _modulew = new ModuleWidget( this, "_modulew" );
+  connect (_modulew, SIGNAL( helpRequest() ), SLOT( slotHelpRequest() ) );
   addWidget( _modulew );
 }
 
@@ -212,8 +205,8 @@ i18n("There are unsaved changes in the active module.\n"
      "Do you want to apply the changes before exiting "
      "the Control Center or discard the changes?"),
                                           i18n("Unsaved Changes"),
-                                          i18n("&Apply"),
-                                          i18n("&Discard"));
+                                          KStdGuiItem::apply(),
+                                          KStdGuiItem::discard());
       if (res == KMessageBox::Yes)
         _module->module()->applyClicked();
       if (res == KMessageBox::Cancel)
@@ -256,3 +249,9 @@ void DockContainer::quickHelpChanged()
   if (_module && _module->module())
 	emit newModule(_module->module()->caption(),  _module->docPath(), _module->module()->quickHelp());
 }
+
+void DockContainer::slotHelpRequest()
+{
+  if (_module && _module->module())
+        _module->module()->helpClicked();
+}
