@@ -13,8 +13,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <unistd.h>
@@ -273,7 +273,7 @@ void KDMUsersWidget::changeUserPix(const QString &pix)
     if (user == m_defaultText)
     {
        user = ".default";
-       if (KMessageBox::questionYesNo(this, i18n("Save image as default image?"))
+       if (KMessageBox::questionYesNo(this, i18n("Save image as default image?"),QString::null,KStdGuiItem::save(),KStdGuiItem::cancel())
             != KMessageBox::Yes)
           return;
     }
@@ -418,7 +418,8 @@ void KDMUsersWidget::slotAddUsers(const QMap<QString,int> &users)
 	      setOn(selectedUsers.find(*name) != selectedUsers.end());
       (new QCheckListItem(optoutlv, *name, QCheckListItem::CheckBox))->
 	      setOn(hiddenUsers.find(*name) != hiddenUsers.end());
-      usercombo->insertItem(*name);
+      if ((*name)[0] != '@')
+        usercombo->insertItem(*name);
     }
     optinlv->sort();
     optoutlv->sort();
