@@ -62,7 +62,7 @@ void KKeyModule::init( bool isGlobal, bool _bSeriesOnly, bool bSeriesNone )
 #define NOSLOTS
 #define KShortcuts KAccelShortcuts
 #include "../../kwin/kwinbindings.cpp"
-#include "../../kicker/core/kickerbindings.cpp"
+#include "../../kicker/kicker/core/kickerbindings.cpp"
 #include "../../kicker/taskbar/taskbarbindings.cpp"
 #include "../../kdesktop/kdesktopbindings.cpp"
 #include "../../klipper/klipperbindings.cpp"
@@ -486,7 +486,7 @@ void KKeyModule::init()
 #define KShortcuts KAccelShortcuts
 #include "../../klipper/klipperbindings.cpp"
 #include "../../kwin/kwinbindings.cpp"
-#include "../../kicker/core/kickerbindings.cpp"
+#include "../../kicker/kicker/core/kickerbindings.cpp"
 #include "../../kicker/taskbar/taskbarbindings.cpp"
 #include "../../kdesktop/kdesktopbindings.cpp"
 #include "../../kxkb/kxkbbindings.cpp"
