@@ -30,7 +30,6 @@
 #include "FontEngine.h"
 #include "kxftconfig.h"
 #include <ksavefile.h>
-#include <qdatastream.h>
 #include <qtextstream.h>
 #include <fstream>
 #include <string.h>
@@ -61,14 +60,13 @@ CXConfig::CXConfig(EType type, const QString &file)
     readConfig();
 }
 
-bool CXConfig::configureDir(const QString &dir, CFontEngine &fe)
+bool CXConfig::configureDir(const QString &dir)
 {
-    bool status=createFontsDotDir(dir, fe);
-#ifdef HAVE_FONT_ENC
-    if(status)
-        status=CEncodings::createEncodingsDotDir(dir);
-#endif
-    return status;
+    //
+    // On systems without mkfontscale, the following will fail, so cant base
+    // return value upon that - hence only check return value of mkfontdir
+    Misc::doCmd("mkfontscale", QFile::encodeName(dir));
+    return Misc::doCmd("mkfontdir", QFile::encodeName(dir));
 }
 
 bool CXConfig::readConfig()
@@ -446,7 +444,6 @@ bool CXConfig::processX11(bool read)
 
                                 if(NULL==findPath(path))
                                     itsPaths.append(new TPath(path, unscaled, TPath::getType(path)));
-
                             }
 
                         if(read)
@@ -456,9 +453,9 @@ bool CXConfig::processX11(bool read)
                             Misc::createBackup(itsFileName);
 
                             KSaveFile   out(itsFileName);
-                            QDataStream *stream=out.dataStream();
+                            FILE        *fstream=out.fstream();
 
-                            if(stream)
+                            if(fstream)
                             {
                                 char  *from=buffer,
                                       *modStart=NULL,
@@ -478,38 +475,40 @@ bool CXConfig::processX11(bool read)
 
                                 if(!foundFt && modStart && modEnd && modStart<filesStart) // Then write mod section first...
                                 {
-                                    stream->writeBytes(from, modEnd-from);
+                                    fwrite(from, 1, modEnd-from, fstream);
                                     if(!foundFt)
-                                        *stream << "    Load \"freetype\"\n";    // CPD TODO: Which is better xtt of freetype? Perhaps check locale?
-                                    stream->writeBytes(modEnd, endSectionMarkerLen);
+                                        fputs("    Load \"freetype\"\n", fstream);    // CPD TODO: Which is better xtt of freetype? Perhaps check locale?
+                                    fwrite(modEnd, 1, endSectionMarkerLen, fstream);
                                     from=modEnd+endSectionMarkerLen;
                                 }
 
-                                stream->writeBytes(from, filesEnd-from);
+                                fwrite(from, 1, filesEnd-from, fstream);
 
                                 for(path=itsPaths.first(); path; path=itsPaths.next())
                                     if(TPath::DIR!=path->type || Misc::dExists(path->dir))
                                     {
-                                        *stream << "    FontPath \t\"";
-                                        *stream << QFile::encodeName(Misc::xDirSyntax(path->dir));
+                                        QCString cPath(QFile::encodeName(Misc::xDirSyntax(path->dir)));
+
+                                        fputs("    FontPath \t\"", fstream);
+                                        fwrite(cPath.data(), 1, cPath.length(), fstream);
                                         if(path->unscaled)
-                                            *stream << UNSCALED;
-                                        *stream << "\"\n";
+                                            fputs(UNSCALED, fstream);
+                                        fputs("\"\n", fstream);
                                     }
 
-                                stream->writeBytes(filesEnd, endSectionMarkerLen);
+                                fwrite(filesEnd, 1, endSectionMarkerLen, fstream);
                                 from=filesEnd+endSectionMarkerLen;
 
                                 if(!foundFt && modStart && modEnd && modStart>filesStart) // Then write mod section last...
                                 {
-                                    stream->writeBytes(from, modEnd-from);
+                                    fwrite(from, 1, modEnd-from, fstream);
                                     if(!foundFt)
-                                        *stream << "    Load \"freetype\"\n";
-                                    stream->writeBytes(modEnd, endSectionMarkerLen);
+                                        fputs("    Load \"freetype\"\n", fstream);
+                                    fwrite(modEnd, 1, endSectionMarkerLen, fstream);
                                     from=modEnd+endSectionMarkerLen;
                                 }
                                 if(((unsigned int)(from-buffer))<size)
-                                    stream->writeBytes(from, size-(from-buffer));
+                                    fwrite(from, 1, size-(from-buffer), fstream);
                                 ok=true;
                             }
                         }
@@ -694,30 +693,32 @@ bool CXConfig::processXfs(bool read)
                                                     {
                                                         Misc::createBackup(itsFileName);
 
-                                                        KSaveFile   out(itsFileName);
-                                                        QDataStream *stream=out.dataStream();
+                                                        KSaveFile out(itsFileName);
+                                                        FILE      *fstream=out.fstream();
 
-                                                        if(stream)
+                                                        if(fstream)
                                                         {
                                                             bool  first=true;
                                                             TPath *p=NULL;
 
-                                                            stream->writeBytes(buffer, cat-buffer);
-                                                            *stream << ' ';
+                                                            fwrite(buffer, 1, cat-buffer, fstream);
+                                                            fputc(' ', fstream);
                                                             for(p=itsPaths.first(); p; p=itsPaths.next())
                                                                 if(Misc::dExists(p->dir))
                                                                 {
+                                                                    QCString cPath(QFile::encodeName(Misc::xDirSyntax(p->dir)));
+
                                                                     if(!first)
                                                                     {
-                                                                        *stream << ',';
-                                                                        *stream << '\n';
+                                                                        fputc(',', fstream);
+                                                                        fputc('\n', fstream);
                                                                     }
-                                                                    *stream << QFile::encodeName(Misc::xDirSyntax(p->dir));
+                                                                    fwrite(cPath.data(), 1, cPath.length(), fstream);
                                                                     if(p->unscaled)
-                                                                        *stream << UNSCALED;
+                                                                        fputs(UNSCALED, fstream);
                                                                     first=false;
                                                                 }
-                                                            stream->writeBytes(cat, size-(cat-buffer));
+                                                            fwrite(cat, 1, size-(cat-buffer), fstream);
                                                             ok=true;
                                                         }
                                                     }
@@ -742,175 +743,6 @@ bool CXConfig::processXfs(bool read)
     return ok;
 }
 
-bool CXConfig::createFontsDotDir(const QString &dir, CFontEngine &fe)
-{
-    QString ds(Misc::dirSyntax(dir));
-    QDir    d(ds);
-    bool    status=true;
-
-    if(d.isReadable())
-    {
-        CFontsFile          fd(QFile::encodeName(QString(ds+"fonts.dir"))),
-                            fs(QFile::encodeName(QString(ds+"fonts.scale")));
-        const QFileInfoList *files=d.entryInfoList();
-        QStringList         fdir,
-                            fscale;
-        bool                addedFd=false,
-                            addedFs=false;
- 
-        if(files)
-        {
-            QFileInfoListIterator it(*files);
-            QFileInfo             *fInfo;
- 
-            for(; NULL!=(fInfo=it.current()); ++it)
-            {
-                QCString          cName(QFile::encodeName(fInfo->fileName()));
-                const QStringList *origfd=NULL,
-                                  *origfs=NULL;
-
-                if("."!=fInfo->fileName() && ".."!=fInfo->fileName())
-                {
-                    origfd=fd.getXlfds(fInfo->fileName());
-                    origfs=fs.getXlfds(fInfo->fileName());
-
-                    if(origfd)
-                        fdir+=*origfd;
-                    else if(origfs)
-                        fdir+=*origfs;
-
-                    if(origfs)
-                        fscale+=*origfs;
-
-                    if(!origfd && !origfs)
-                    {
-                        int face=0,
-                            numFaces=0;
-
-                        do
-                        {
-                            if(fe.openFont(fInfo->filePath(), face))
-                            {
-                                if(CFontEngine::BITMAP!=fe.getType())
-                                {
-                                     QStringList encodings=fe.getEncodings();
-
-                                     numFaces=fe.getNumFaces();   // Only really for TTC files..
-
-                                     if(encodings.count())
-                                     {
-                                         QCString xlfd;
-                                         QString  family=fe.getFamilyName();
-
-                                         if(face>0)  // Again, only really for TTC files...
-                                         {
-                                             QCString fc;
-                                             fc.setNum(face);
-                                             xlfd+=':';
-                                             xlfd+=fc;
-                                             xlfd+=':';
-                                         }
-
-                                         xlfd+=QFile::encodeName(fInfo->fileName());
-                                         xlfd+=" -";
-                                         xlfd+=fe.getFoundry().latin1();
-                                         xlfd+="-";
-                                         xlfd+=family.latin1();
-                                         xlfd+="-";
-                                         xlfd+=CFontEngine::weightStr(fe.getWeight()).latin1();
-                                         xlfd+="-";
-                                         xlfd+=CFontEngine::italicStr(fe.getItalic()).latin1();
-                                         xlfd+="-";
-                                         xlfd+=CFontEngine::widthStr(fe.getWidth()).latin1();
-                                         xlfd+="-";
-                                         if(!fe.getAddStyle().isEmpty())
-                                             xlfd+=fe.getAddStyle().latin1();
-                                         //CPD: TODO Bitmap only ttfs!!!
-                                         xlfd+="-0-0-0-0-";
-
-                                         QStringList::Iterator it;
-
-                                         for(it=encodings.begin(); it!=encodings.end(); ++it)
-                                         {
-                                             QCString entry(xlfd);
-
-                                             // Taken from ttmkfdir...
-                                             if((*it).find("jisx")!=-1 || (*it).find("gb2312")!=-1 ||
-                                                (*it).find("big5")!=-1 || (*it).find("ksc")!=-1)
-                                                 entry+='c';
-                                             else
-                                                 entry+=CFontEngine::spacingStr(fe.getSpacing()).latin1();
-
-                                             entry+="-0-";
-                                             entry+=(*it).latin1();
-
-                                             if(-1==fscale.findIndex(entry))
-                                                 fscale.append(entry);
-                                             if(-1==fdir.findIndex(entry))
-                                                 fdir.append(entry);
-                                             addedFd=addedFs=true;
-                                         }
-                                     }
-                                }
-                                else
-                                {
-                                    QCString entry(QFile::encodeName(fInfo->fileName()));
-        
-                                    entry+=" ";
-                                    entry+=fe.getXlfdBmp().latin1();
-                                    fdir.append(entry);
-                                    addedFd=true;
-                                }
-                                fe.closeFont();
-                             }
-                         }
-                         while(++face<numFaces);
-                      }
-                }
-            }
-        }
-
-        //
-        // Only output if we have added something, or the enumber of Xlfds is different (would mean a font was
-        // removed)
-        if(addedFd || fdir.count()!=fd.xlfdCount() || (0==fdir.count() && 0!=fd.xlfdCount()))
-        {
-
-            KSaveFile   out(ds+"fonts.dir");
-            QTextStream *stream=out.textStream();
-
-            if(stream)
-            {
-                QStringList::Iterator sIt;
-
-                *stream << fdir.count() << endl;
-                for(sIt=fdir.begin(); sIt!=fdir.end(); ++sIt)
-                    *stream << (*sIt).local8Bit() << endl;
-            }
-            else
-                status=false;
-        }
-
-        if(status && (addedFs || fscale.count()!=fs.xlfdCount()))
-        {
-            KSaveFile   out(ds+"fonts.scale");
-            QTextStream *stream=out.textStream();
-
-            if(stream)
-            {
-                QStringList::Iterator sIt;
-
-                *stream << fscale.count() << endl;
-                for(sIt=fscale.begin(); sIt!=fscale.end(); ++sIt)
-                    *stream << (*sIt).local8Bit() << endl;
-            }
-            else
-                status=false;
-        }
-    }
-
-    return status;
-}
 
 CXConfig::TPath::EType CXConfig::TPath::getType(const QString &d)
 {
@@ -925,107 +757,4 @@ CXConfig::TPath::EType CXConfig::TPath::getType(const QString &d)
                    : DIR;
 }
 
-CXConfig::CFontsFile::CFontsFile(const char *file)
-                    : itsXlfdCount(0)
-{
-    std::ifstream f(file);
-
-    itsEntries.setAutoDelete(true);
-
-    if(f)
-    {
-        const int constMaxLine=512;
-
-        char   line[constMaxLine+1];
-        TEntry *current=NULL;
-
-        f.getline(line, constMaxLine); // Ignore # of entries
-
-        while(!f.eof())
-        {
-            f.getline(line, constMaxLine);
-            if(!f.eof())
-            {
-                // Assume entries are:
-                //    <file>.<ext>   -<xlfd>
-                char *dash=strstr(line, " -");
-
-                if(!dash)
-                    dash=strstr(line, "\t-");
-
-                if(dash)
-                {
-                    itsXlfdCount++;
-
-                    QString xlfd(&dash[1]);
-
-                    *dash='\0';
-
-                    //
-                    // Check for ttc files, and ttcap entries...
-                    QString fname(QString(line).stripWhiteSpace()), 
-                            mod;
-                    int     c1=fname.findRev(':');
-
-                    if(-1!=c1)
-                    {
-                        mod=fname.left(c1+1);
-                        fname.remove(0, c1+1);
-                    }
-
-                    TEntry *entry=getEntry(&current, fname);
-
-                    if(entry)
-                        if(mod.isEmpty())
-                            entry->xlfds.append(entry->filename+" "+xlfd);
-                        else
-                            entry->xlfds.append(mod+entry->filename+" "+xlfd);
-                }
-            }
-        }
-        f.close();
-    }
-}
-
-const QStringList * CXConfig::CFontsFile::getXlfds(const QString &fname)
-{
-    TEntry *entry=findEntry(fname);
-
-    return entry ? &entry->xlfds : NULL;
-}
-
-CXConfig::CFontsFile::TEntry * CXConfig::CFontsFile::findEntry(const QString &fname)
-{
-    TEntry *entry=NULL;
-
-    for(entry=itsEntries.first(); entry; entry=itsEntries.next())
-        if(entry->filename==fname)
-            break;
-
-    return entry;
-}
-
-CXConfig::CFontsFile::TEntry * CXConfig::CFontsFile::getEntry(TEntry **current, const QString &filename)
-{
-    //
-    // See if its the current one...
-    if(*current && (*current)->filename==filename)
-        return *current;
-
-    //
-    // See if its already known...
-    TEntry *entry=findEntry(filename);
-
-    //
-    // If not found, then create a new entry
-    if(!entry)
-    {
-        entry=new TEntry(filename);
-        itsEntries.append(entry);
-    }
-
-    *current=entry;
-    return entry;
-}
-
 }
