@@ -487,7 +487,7 @@ void PositionTab::panelDimensionsChanged()
 
 void PositionTab::slotBGPreviewReady(int)
 {
-    m_pretendDesktop->setBackgroundPixmap(*m_desktopPreview->pixmap());
+    m_pretendDesktop->setBackgroundPixmap(m_desktopPreview->pixmap());
 #if 0
     KPixmap pm;
     if (QPixmap::defaultDepth() < 15)
@@ -687,10 +687,7 @@ void PositionTab::showIdentify()
     for(int s=0; s < QApplication::desktop()->numScreens();s++)
     {
 
-        QLabel *screenLabel = new QLabel(0,"Screen Identify", WStyle_StaysOnTop | WDestructiveClose | WStyle_Customize | WStyle_NoBorder);
-
-        KWin::setState( screenLabel->winId(), NET::Modal | NET::Sticky | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
-        KWin::setType( screenLabel->winId(), NET::Override );
+        QLabel *screenLabel = new QLabel(0,"Screen Identify", WDestructiveClose | WStyle_Customize | WX11BypassWM );
 
         QFont identifyFont(KGlobalSettings::generalFont());
         identifyFont.setPixelSize(100);
