@@ -60,7 +60,7 @@ SMBRoOptions::SMBRoOptions(QWidget *parent)
    layout->addWidget(m_workgroupLe,3,1);
 
    m_showHiddenShares=new QCheckBox(i18n("Show hidden shares"),this);
-   layout->addMultiCellWidget(m_showHiddenShares,4,4,0,1);*/
+   layout->addMultiCellWidget(m_showHiddenShares,4,4,0,1);
 
    m_encodingList = new KComboBox( false, this );
    QStringList _strList = KGlobal::charsets()->availableEncodingNames();
@@ -69,6 +69,7 @@ SMBRoOptions::SMBRoOptions(QWidget *parent)
    label = new QLabel( m_encodingList, i18n( "MS Windows encoding:" ), this );
    layout->addWidget( label, 3, 0 );
    layout->addWidget( m_encodingList, 3, 1 );
+   */
 
    layout->addWidget(new QWidget(this),4,0);
 
@@ -76,7 +77,7 @@ SMBRoOptions::SMBRoOptions(QWidget *parent)
    connect(m_userLe, SIGNAL(textChanged(const QString&)), this, SLOT(changed()));
    connect(m_passwordLe, SIGNAL(textChanged(const QString&)), this, SLOT(changed()));
 //   connect(m_workgroupLe, SIGNAL(textChanged(const QString&)), this, SLOT(changed()));
-   connect( m_encodingList, SIGNAL( activated( const QString & ) ), this , SLOT( changed() ) );
+//   connect( m_encodingList, SIGNAL( activated( const QString & ) ), this , SLOT( changed() ) );
 
    layout->setRowStretch(4, 1);
 
@@ -98,9 +99,9 @@ void SMBRoOptions::load()
 //   m_workgroupLe->setText(cfg->readEntry("Workgroup"));
 //   m_showHiddenShares->setChecked(cfg->readBoolEntry("ShowHiddenShares",false));
 
-   QStringList _strList = KGlobal::charsets()->availableEncodingNames();
-   QString m_encoding = QTextCodec::codecForLocale()->name();
-   m_encodingList->setCurrentItem( _strList.findIndex( cfg->readEntry( "Encoding", m_encoding.lower() ) ) );
+//   QStringList _strList = KGlobal::charsets()->availableEncodingNames();
+//   QString m_encoding = QTextCodec::codecForLocale()->name();
+//   m_encodingList->setCurrentItem( _strList.findIndex( cfg->readEntry( "Encoding", m_encoding.lower() ) ) );
 
    // unscramble
    QString scrambled = cfg->readEntry( "Password" );
@@ -129,7 +130,7 @@ void SMBRoOptions::save()
    cfg->writeEntry( "User", m_userLe->text());
 //   cfg->writeEntry( "Workgroup", m_workgroupLe->text());
 //   cfg->writeEntry( "ShowHiddenShares", m_showHiddenShares->isChecked());
-   cfg->writeEntry( "Encoding", m_encodingList->currentText() );
+//   cfg->writeEntry( "Encoding", m_encodingList->currentText() );
 
    //taken from Nicola Brodu's smb ioslave
    //it's not really secure, but at
