@@ -8,20 +8,17 @@
  *                                                                         *
  ***************************************************************************/
 
-
-
-
-#include <qlayout.h>
 #include <qgroupbox.h>
-#include <qsplitter.h>
-#include <qlistview.h>
-#include <qtextview.h>
 #include <qheader.h>
+#include <qlayout.h>
+#include <qlistview.h>
+#include <qsplitter.h>
+#include <qtextview.h>
 #include <qtimer.h>
 
-#include <kgenericfactory.h>
 #include <kaboutdata.h>
 #include <kdialog.h>
+#include <kgenericfactory.h>
 
 #include "usbdevices.h"
 #include "kcmusb.moc"
@@ -34,6 +31,9 @@ USBViewer::USBViewer(QWidget *parent, const char *name, const QStringList &)
 {
   setButtons(Help);
 
+  setQuickHelp( i18n("<h1>USB Devices</h1> This module allows you to see"
+     " the devices attached to your USB bus(es)."));
+
   QVBoxLayout *vbox = new QVBoxLayout(this, 0, KDialog::spacingHint());
   QGroupBox *gbox = new QGroupBox(i18n("USB Devices"), this);
   gbox->setColumnLayout( 0, Qt::Horizontal );
@@ -79,12 +79,6 @@ USBViewer::USBViewer(QWidget *parent, const char *name, const QStringList &)
   load();
 }
 
-
-USBViewer::~USBViewer()
-{
-}
-
-
 void USBViewer::load()
 {
   _items.clear();
@@ -123,7 +117,7 @@ void USBViewer::refresh()
   QIntDict<QListViewItem> new_items;
 
   if (!USBDevice::parse("/proc/bus/usb/devices"))
-    USBDevice::parse("/proc/bus/usb/devices_please-use-sysfs-instead");
+    USBDevice::parseSys("/sys/bus/usb/devices");
 
   int level = 0;
   bool found = true;
@@ -196,19 +190,3 @@ void USBViewer::selectionChanged(QListViewItem *item)
 }
 
 
-void USBViewer::save()
-{
-}
-
-
-void USBViewer::defaults()
-{
-}
-
-
-QString USBViewer::quickHelp() const
-{
-  return i18n("<h1>USB Devices</h1> This module allows you to see"
-     " the devices attached to your USB bus(es).");
-}
-
