@@ -14,14 +14,19 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
 
 #ifndef DEFCMDS_H
 #define DEFCMDS_H
 
-#define efax_default_cmd	"/usr/bin/fax 'NAME=\"%user\"' DEV=%dev PAGE=%page 'FROM=\"%from\"' send %res %number %files"
-#define hylafax_default_cmd	"/usr/bin/sendfax -h '%server' %res -d '%name'@'%number' -x '%enterprise' -c '%comment' -f '%user' %files"
+#include <qstring.h>
+
+#define efax_default_cmd    "%exe_fax %user_{NAME=\"@@\"} %dev_{DEV=@@} PAGE=%page %from_{FROM=@@} %res_{?\?-l} send %number %files"
+#define hylafax_default_cmd "%exe_sendfax %cover_{?\?-n} %server_h %res_{-m?\?-l} %subject_r %time_a %enterprise_x %comment_c %email_f %from_W %page_s -d %name_{@@@}%number %files"
+#define mgetty_default_cmd  "%exe_faxspool %user_F %email_f %name_D %time_t %number %files %res_{?\?-n} %cover_{?\?-C -}"
+
+QString defaultCommand(const QString& cmd);
 
 #endif
