@@ -23,6 +23,7 @@
 #include "faxctrl.h"
 #include "configdlg.h"
 
+#include <qcheckbox.h>
 #include <qlineedit.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -108,7 +109,10 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 	m_tray->setPixmap(SmallIcon("kdeprintfax"));
 	m_tray->show();
 	m_comment = new QTextEdit(mainw);
-	m_comment->setWordWrap(QTextEdit::NoWrap);
+// I don't understand why anyone would want to turn off word wrap. It makes
+// the text hard to read and write. It provides no benefit. Therefore,
+// I commented out the next line. [Ray Lischner]
+//	m_comment->setWordWrap(QTextEdit::NoWrap);
 	m_comment->setLineWidth(1);
 	m_commentlabel->setBuddy(m_comment);
 	QLabel	*m_timelabel = new QLabel(i18n("Sched&ule:"), mainw);
@@ -121,11 +125,13 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 	m_time->setTime(QTime::currentTime());
 	m_time->setEnabled(false);
 	connect(m_timecombo, SIGNAL(activated(int)), SLOT(slotTimeComboActivated(int)));
+	m_cover = new QCheckBox(i18n("Send Co&ver Sheet"), mainw);
+	connect(m_cover, SIGNAL(toggled(bool)), SLOT(slotCoverToggled(bool)));
 	m_subject = new QLineEdit( mainw );
 	QLabel *m_subjectlabel = new QLabel( i18n( "Su&bject:" ), mainw );
 	m_subjectlabel->setBuddy( m_subject );
 
-	QGridLayout	*l0 = new QGridLayout(mainw, 9, 2, 10, 5);
+	QGridLayout	*l0 = new QGridLayout(mainw, 10, 2, 10, 5);
 	l0->setColStretch(1,1);
 	l0->addWidget(m_filelabel, 0, 0, Qt::AlignLeft|Qt::AlignTop);
 	QHBoxLayout *l2 = new QHBoxLayout( 0, 0, 10 );
@@ -150,13 +156,14 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 	l6->addStretch( 1 );
 	l0->addMultiCellWidget( m_line2, 5, 5, 0, 1 );
 	l0->addRowSpacing( 5, 10 );
-	l0->addWidget( m_subjectlabel, 6, 0 );
-	l0->addWidget( m_subject, 6, 1 );
-	l0->addWidget(m_commentlabel, 7, 0, Qt::AlignTop|Qt::AlignLeft);
-	l0->addWidget(m_comment, 7, 1);
-	l0->addWidget(m_timelabel, 8, 0);
+	l0->addWidget( m_cover, 6, 1 );
+	l0->addWidget( m_subjectlabel, 7, 0 );
+	l0->addWidget( m_subject, 7, 1 );
+	l0->addWidget(m_commentlabel, 8, 0, Qt::AlignTop|Qt::AlignLeft);
+	l0->addWidget(m_comment, 8, 1);
+	l0->addWidget(m_timelabel, 9, 0);
 	QHBoxLayout	*l1 = new QHBoxLayout(0, 0, 5);
-	l0->addLayout(l1, 8, 1);
+	l0->addLayout(l1, 9, 1);
 	l1->addWidget(m_timecombo, 1);
 	l1->addWidget(m_time, 0);
 
@@ -373,6 +380,11 @@ QString KdeprintFax::comment() const
 	return m_comment->text();
 }
 
+bool KdeprintFax::cover() const
+{
+	return m_cover->isChecked();
+}
+
 QString KdeprintFax::subject() const
 {
 	return m_subject->text();
@@ -413,7 +425,10 @@ void KdeprintFax::slotConfigure()
 void KdeprintFax::updateState()
 {
 	QString	cmd = m_faxctrl->faxCommand();
-	m_comment->setEnabled(cmd.find("%comment") != -1);
+	m_cover->setEnabled(cmd.find("%cover") != -1);
+	if ( !m_cover->isEnabled() )
+		m_cover->setChecked(false);
+	m_comment->setEnabled(cmd.find("%comment") != -1 && m_cover->isChecked());
 	//m_comment->setPaper(m_comment->isEnabled() ? colorGroup().brush(QColorGroup::Base) : colorGroup().brush(QColorGroup::Background));
 	if (!m_comment->isEnabled())
 	{
@@ -436,7 +451,7 @@ void KdeprintFax::updateState()
 	else
 		m_timecombo->setEnabled( true );
 	/*m_name->setEnabled( cmd.find( "%name" ) != -1 );*/
-	m_subject->setEnabled( cmd.find( "%subject" ) != -1 );
+	m_subject->setEnabled( cmd.find( "%subject" ) != -1 && m_cover->isChecked() );
 	statusBar()->changeItem(m_faxctrl->faxSystem(), 2);
 }
 
@@ -481,6 +496,15 @@ void KdeprintFax::slotMoveDown()
 	}
 }
 
+/** The user or program toggled the "Cover Sheet" check box.
+ * Update the state of the other controls to reflect the
+ * new status.
+ */
+void KdeprintFax::slotCoverToggled(bool)
+{
+	updateState();
+}
+
 void KdeprintFax::slotCurrentChanged()
 {
 	int index = m_files->currentItem();
