@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "kdiconview.h"
@@ -22,6 +22,7 @@
 #include "desktop.h"
 #include "kdesktopsettings.h"
 
+#include <kio/paste.h>
 #include <kaccel.h>
 #include <kapplication.h>
 #include <kcolordrag.h>
@@ -29,6 +30,7 @@
 #include <kdesktopfile.h>
 #include <kdirlister.h>
 #include <kglobalsettings.h>
+#include <kpropertiesdialog.h>
 #include <klocale.h>
 #include <konqbookmarkmanager.h>
 #include <konq_defaults.h>
@@ -60,41 +62,80 @@
 // for multihead
 extern int kdesktop_screen_number;
 
-#define saveIconPosition(config, _x, _y, iconArea) \
-do { \
-    int x = _x; int y = _y; \
-    x -= iconArea.x(); \
-    y -= iconArea.y(); \
-    if (x > ((iconArea.width()*3) / 4)) x -= iconArea.width(); \
-    if (y > ((iconArea.height()*3) / 4)) y -= iconArea.height(); \
-    (config)->writeEntry("X", x); (config)->writeEntry("Y", y); \
-    (config)->writeEntry(X_w, x); (config)->writeEntry(Y_h, y); \
-} while(false)
+// -----------------------------------------------------------------------------
 
-#define readIconPosition(config, x, y, iconArea) \
-do { \
-    x = (config)->readNumEntry( X_w, -9999 ); \
-    if ( x == -9999 ) x = (config)->readNumEntry( "X" ); \
-    if (x < 0) x += iconArea.width(); \
-    y = (config)->readNumEntry( Y_h, -9999 ); \
-    if ( y == -9999 ) y = (config)->readNumEntry( "Y" ); \
-    if (y < 0) y += iconArea.height(); \
-    x += iconArea.x(); \
-    y += iconArea.y(); \
-} while(false)
-
-static QRect desktopRect() {
+QRect KDIconView::desktopRect()
+{
     return ( kdesktop_screen_number == 0 )
             ? QApplication::desktop()->geometry() // simple case, or xinerama
             : QApplication::desktop()->screenGeometry( kdesktop_screen_number ); // multi-head
 }
 
+// -----------------------------------------------------------------------------
+
+void KDIconView::saveIconPosition(KSimpleConfig *config, int x, int y)
+{
+  // save the icon position in absolute coordinates
+  config->writeEntry("Xabs", x);
+  config->writeEntry("Yabs", y);
+
+  // save also mentioning desktop size
+  QRect desk = desktopRect();
+  QString sizeStr = QString( "_%1x%2" ).arg(desk.width()).arg(desk.height());
+
+  config->writeEntry("Xabs" + sizeStr, x);
+  config->writeEntry("Yabs" + sizeStr, y);
+}
+
+// -----------------------------------------------------------------------------
+
+void KDIconView::readIconPosition(KSimpleConfig *config, int &x, int &y)
+{
+  // check if we have the position for the current desktop size
+  QRect desk = desktopRect();
+  QString sizeStr = QString( "_%1x%2" ).arg(desk.width()).arg(desk.height());
+
+  x = config->readNumEntry("Xabs" + sizeStr, -99999);
+
+  if ( x != -99999 )
+    y = config->readNumEntry("Yabs" + sizeStr);
+  else
+  {
+    // not found; use the resolution independent position
+    x = config->readNumEntry("Xabs", -99999);
+
+    if ( x != -99999 )
+      y = config->readNumEntry("Yabs");
+    else  // for compatibility, read the old iconArea-relative-position
+    {
+      // problem here: when reading a position before we know the correct
+      // desktopIconsArea, the relative position do not make sense
+      // workaround: use desktopRect() as the allowed size
+
+      QRect desk = desktopRect();
+      QString X_w = QString("X %1").arg(desk.width() );
+      QString Y_h = QString("Y %1").arg(desk.height());
+
+      x = config->readNumEntry(X_w, -99999);
+      if ( x != -99999 ) x = config->readNumEntry("X");
+      if ( x < 0 ) x += desk.width();
+
+      y = config->readNumEntry(Y_h, -99999);
+      if ( y != -99999 ) y = config->readNumEntry("Y");
+      if ( y < 0 ) y += desk.height();
+    }
+  }
+}
+
+// -----------------------------------------------------------------------------
+
 KDIconView::KDIconView( QWidget *parent, const char* name )
     : KonqIconViewWidget( parent, name, WResizeNoErase, true ),
       m_actionCollection( this, "KDIconView::m_actionCollection" ),
       m_accel( 0L ),
       m_bNeedRepaint( false ),
       m_bNeedSave( false ),
+      m_autoAlign( false ),
       m_hasExistingPos( false ),
       m_bEditableDesktopIcons( kapp->authorize("editable_desktop_icons") ),
       m_bShowDot( false ),
@@ -105,9 +146,11 @@ KDIconView::KDIconView( QWidget *parent, const char* name )
       m_lastDeletedIconPos(),
       m_eSortCriterion( NameCaseInsensitive ),
       m_bSortDirectoriesFirst( true ),
-      m_itemsAlwaysFirst()
+      m_itemsAlwaysFirst(),
+      m_gotIconsArea(false)
 {
     setResizeMode( Fixed );
+    setIconArea( desktopRect() );  // the default is the whole desktop
 
     // Initialise the shadow data objects...
     m_shadowEngine = new KShadowEngine(new KDesktopShadowSettings(KGlobal::config()));
@@ -146,10 +189,6 @@ KDIconView::KDIconView( QWidget *parent, const char* name )
        setAcceptDrops(false);
        viewport()->setAcceptDrops(false);
     }
-
-#if (QT_VERSION-0 >= 0x030200) // XRANDR support
-    connect( kapp->desktop(), SIGNAL( resized( int )), SLOT( desktopResized()));
-#endif
 }
 
 KDIconView::~KDIconView()
@@ -216,7 +255,7 @@ void KDIconView::initDotDirectories()
                  if (!m_dotDirectory->hasKey( X_w ))
                  {
                     int x,y;
-                    readIconPosition(&dotDir, x, y, iconArea());
+                    readIconPosition(&dotDir, x, y);
                     m_dotDirectory->writeEntry( X_w, x );
                     m_dotDirectory->writeEntry( Y_h, y ); // Not persistant!
                  }
@@ -288,7 +327,7 @@ void KDIconView::initConfig( bool init )
                     Q_ASSERT( serv != 0L );
                     if ( serv )
                     {
-                        setIcons( iconSize( ), serv->property("MimeTypes").toStringList().join(";" ) /* revert no-longer wanted previews to icons */ );
+                        setIcons( iconSize( ), serv->property("MimeTypes").toStringList() /* revert no-longer wanted previews to icons */ );
                     }
                 }
             }
@@ -332,22 +371,7 @@ void KDIconView::start()
     // Start the directory lister !
     m_dirLister->setShowingDotFiles( m_bShowDot );
     kapp->allowURLAction("list", KURL(), url());
-    m_dirLister->openURL( url() );
-
-    // Gather the list of directories to merge into the desktop
-    // (the main URL is desktopURL(), no need for it in the m_mergeDirs list)
-    m_mergeDirs.clear();
-    for ( QStringList::ConstIterator it = m_desktopDirs.begin() ; it != m_desktopDirs.end() ; ++it )
-    {
-        kdDebug(1204) << "KDIconView::start found merge dir " << *it << endl;
-        KURL u;
-        u.setPath( *it );
-        m_mergeDirs.append( u );
-        // And start listing this dir right now
-        kapp->allowURLAction("list", KURL(), u);
-        m_dirLister->openURL( u, true );
-    }
-    configureMedia();
+    startDirLister();
     createActions();
 }
 
@@ -407,7 +431,11 @@ void KDIconView::createActions()
         KAction *pasteTo = KStdAction::paste( this, SLOT( slotPopupPasteTo() ), &m_actionCollection, "pasteto" );
         pasteTo->setEnabled( false ); // only enabled during popupMenu()
 
+        KShortcut reloadShortcut = KStdAccel::shortcut(KStdAccel::Reload);
+        new KAction( i18n( "&Reload" ), "reload", reloadShortcut, this, SLOT( refreshIcons() ), &m_actionCollection, "reload" );
+
         (void) new KAction( i18n( "&Rename" ), /*"editrename",*/ Key_F2, this, SLOT( renameSelectedItem() ), &m_actionCollection, "rename" );
+        (void) new KAction( i18n( "&Properties" ), ALT+Key_Return, this, SLOT( slotProperties() ), &m_actionCollection, "properties" );
         KAction* trash = new KAction( i18n( "&Move to Trash" ), "edittrash", Key_Delete, &m_actionCollection, "trash" );
         connect( trash, SIGNAL( activated( KAction::ActivationReason, Qt::ButtonState ) ),
                  this, SLOT( slotTrashActivated( KAction::ActivationReason, Qt::ButtonState ) ) );
@@ -433,16 +461,20 @@ void KDIconView::rearrangeIcons( SortCriterion sc, bool bSortDirectoriesFirst )
 void KDIconView::rearrangeIcons()
 {
     setupSortKeys();
-    sort();
+    sort();  // calls arrangeItemsInGrid() which does not honor iconArea()
+
     if ( m_autoAlign )
-        lineupIcons(  m_bVertAlign ? QIconView::TopToBottom : QIconView::LeftToRight );
+        lineupIcons(  m_bVertAlign ? QIconView::TopToBottom : QIconView::LeftToRight );  // also saves position
     else
-        arrangeItemsInGrid();
-    saveIconPositions();
+    {
+        KonqIconViewWidget::lineupIcons(m_bVertAlign ? QIconView::TopToBottom : QIconView::LeftToRight);
+        saveIconPositions();
+    }
 }
 
 void KDIconView::lineupIcons()
 {
+    if ( !m_gotIconsArea ) return;
     KonqIconViewWidget::lineupIcons();
     saveIconPositions();
 }
@@ -453,7 +485,7 @@ void KDIconView::setAutoAlign( bool b )
 
     // Auto line-up icons
     if ( b ) {
-        lineupIcons();
+            lineupIcons();
         connect( this, SIGNAL( iconMoved() ),
                  this, SLOT( lineupIcons() ) );
     }
@@ -468,14 +500,15 @@ void KDIconView::setAutoAlign( bool b )
     }
 }
 
-void KDIconView::desktopResized()
+void KDIconView::startDirLister()
 {
-    saveIconPositions();
-    resize( kapp->desktop()->size() );
-    slotClear();
+    // if desktop is resized before start() is called (XRandr)
+    if (!m_dirLister) return;
+
     m_dirLister->openURL( url() );
 
-    // list all desktop dirs
+    // Gather the list of directories to merge into the desktop
+    // (the main URL is desktopURL(), no need for it in the m_mergeDirs list)
     m_mergeDirs.clear();
     for ( QStringList::ConstIterator it = m_desktopDirs.begin() ; it != m_desktopDirs.end() ; ++it )
     {
@@ -496,14 +529,18 @@ void KDIconView::lineupIcons(QIconView::Arrangement align)
     setArrangement( m_bVertAlign ? TopToBottom : LeftToRight );
 
     if ( m_autoAlign )
+    {
         KonqIconViewWidget::lineupIcons( align );
+        saveIconPositions();
+    }
     else
-        rearrangeIcons();
+        rearrangeIcons();  // also saves the position
 
     KDesktopSettings::setVertAlign( m_bVertAlign );
     KDesktopSettings::writeConfig();
 }
 
+// Only used for DCOP
 QStringList KDIconView::selectedURLs()
 {
     QStringList seq;
@@ -597,6 +634,16 @@ void KDIconView::wheelEvent( QWheelEvent* e )
     KonqIconViewWidget::wheelEvent( e );
 }
 
+void KDIconView::slotProperties()
+{
+    KFileItemList selectedFiles = selectedFileItems();
+
+    if( selectedFiles.isEmpty() )
+      return;
+
+    (void) new KPropertiesDialog( selectedFiles );
+}
+
 void KDIconView::slotContextMenuRequested(QIconViewItem *_item, const QPoint& _global)
 {
     if (_item)
@@ -872,6 +919,11 @@ bool KDIconView::makeFriendlyText( KFileIVI *fileIVI )
             if (cfg.readListEntry("NotShowIn", ';').contains("KDE"))
                 return false;
         }
+        if (cfg.hasKey("TryExec"))
+        {
+            if (KStandardDirs::findExe( cfg.readEntry( "TryExec" ) ).isEmpty())
+                return false;
+        }
 
         QString name = cfg.readEntry("Name");
         if ( !name.isEmpty() )
@@ -944,10 +996,6 @@ void KDIconView::slotNewItems( const KFileItemList & entries )
     }
     else
     {
-      QRect desk = desktopRect();
-      QString X_w = QString( "X %1" ).arg( desk.width() );
-      QString Y_h = QString( "Y %1" ).arg( desk.height() );
-
       kdDebug(1214) << "slotNewItems : trying to read position from .directory file"<<endl;
       QString group = iconPositionGroupPrefix();
       QString filename = url.fileName();
@@ -960,7 +1008,7 @@ void KDIconView::slotNewItems( const KFileItemList & entries )
         m_dotDirectory->setGroup( group );
         m_hasExistingPos = true;
         int x,y;
-        readIconPosition(m_dotDirectory, x, y, area);
+        readIconPosition(m_dotDirectory, x, y);
 
         kdDebug(1214)<<"slotNewItems() x: "<<x<<" y: "<<y<<endl;
 
@@ -1143,7 +1191,7 @@ void KDIconView::slotCompleted()
     else
     {
         stopImagePreview();
-        setIcons( iconSize(), "" /* stopImagePreview */ );
+        setIcons( iconSize(), "*" /* stopImagePreview */ );
     }
 
     // during first run need to rearrange all icons so default config settings will be used
@@ -1180,7 +1228,13 @@ void KDIconView::slotClipboardDataChanged()
 
     disableIcons( lst );
 
-    bool paste = ( data->format() != 0 ); // check if there's something in the clipboard
+    QString actionText = KIO::pasteActionText();
+    bool paste = !actionText.isEmpty();
+    if ( paste ) {
+        KAction* pasteAction = m_actionCollection.action( "paste" );
+        if ( pasteAction )
+            pasteAction->setText( actionText );
+    }
     slotEnableAction( "paste", paste );
 }
 
@@ -1258,9 +1312,6 @@ void KDIconView::slotAboutToCreate(const QPoint &pos, const QValueList<KIO::Copy
     QValueList<KIO::CopyInfo>::ConstIterator it = files.begin();
     int gridX = gridXValue();
     int gridY = 120; // 120 pixels should be enough for everyone (tm)
-    QRect desk = desktopRect();
-    QString X_w = QString( "X %1" ).arg( desk.width() );
-    QString Y_h = QString( "Y %1" ).arg( desk.height() );
 
     for ( ; it!= files.end() ; ++it )
     {
@@ -1268,7 +1319,7 @@ void KDIconView::slotAboutToCreate(const QPoint &pos, const QValueList<KIO::Copy
         if ((*it).uDest.isLocalFile() && ((*it).uDest.directory() == dir))
         {
            m_dotDirectory->setGroup( iconPositionGroupPrefix() + (*it).uDest.fileName() );
-           saveIconPosition(m_dotDirectory, m_lastDropPos.x(), m_lastDropPos.y(), iconArea());
+           saveIconPosition(m_dotDirectory, m_lastDropPos.x(), m_lastDropPos.y());
            int dX = m_lastDropPos.x() - m_dropPos.x();
            int dY = m_lastDropPos.y() - m_dropPos.y();
            if ((QABS(dX) > QABS(dY)) || (m_lastDropPos.x() + 2*gridX > width()))
@@ -1329,6 +1380,38 @@ void KDIconView::contentsDropEvent( QDropEvent * e )
         setLastIconPosition( e->pos() );
         KonqIconViewWidget::contentsDropEvent( e );
     }
+
+    // Check if any items have been moved outside the desktop area.
+    // If we find any, move them right back in there. (#40418)
+    QRect desk = desktopRect();
+    bool adjustedAnyItems = false;
+    for( QIconViewItem *item = firstItem(); item; item = item->nextItem() )
+    {
+        if( !desk.contains( item->rect(), true ))
+        {
+            QRect r = item->rect();
+
+            if( r.top() < 0 )
+                r.moveTop( 0 );
+            if( r.bottom() > rect().bottom() )
+                r.moveBottom( rect().bottom() );
+            if( r.left() < 0 )
+                r.moveLeft( 0 );
+            if( r.right() > rect().right() )
+                r.moveRight( rect().right() );
+
+            item->move( r.x(), r.y() );
+            adjustedAnyItems = true;
+        }
+    }
+    if( adjustedAnyItems )
+    {
+        // Make sure the viewport isn't unnecessarily resized by now,
+        // then schedule a repaint to remove any garbage pixels.
+        resizeContents( width(), height() );
+        viewport()->update();
+    }
+
     if (QIconDrag::canDecode(e)) {
       emit iconMoved();
       if ( !m_autoAlign )    // if autoAlign, positions were saved in lineupIcons
@@ -1344,24 +1427,47 @@ void KDIconView::viewportWheelEvent( QWheelEvent * e )
 
 void KDIconView::updateWorkArea( const QRect &wr )
 {
-    kdDebug(1204) << "KDIconView::updateWorkArea wr: " << wr.x() << "," << wr.y()
-              << " " << wr.width() << "x" << wr.height() << endl;
+    m_gotIconsArea = true;  // now we have it!
+
+    if ( iconArea() == wr ) return;  // nothing changed; avoid repaint/saveIconPosition ...
+
     QRect oldArea = iconArea();
     setIconArea( wr );
 
+    kdDebug(1204) << "KDIconView::updateWorkArea wr: " << wr.x() << "," << wr.y()
+              << " " << wr.width() << "x" << wr.height() << endl;
+    kdDebug(1204) << "  oldArea:                     " << oldArea.x() << "," << oldArea.y()
+              << " " << oldArea.width() << "x" << oldArea.height() << endl;
+
     if ( m_autoAlign )
         lineupIcons();
     else {
         bool needRepaint = false;
         QIconViewItem* item;
         int dx, dy;
+
         dx = wr.left() - oldArea.left();
         dy = wr.top() - oldArea.top();
+
         if ( dx != 0 || dy != 0 ) {
-            needRepaint = true;
-            for ( item = firstItem(); item; item = item->nextItem() )
-                item->moveBy( dx, dy );
+          if ( (dx > 0) || (dy > 0) ) // the iconArea was shifted right/down; less space now
+              for ( item = firstItem(); item; item = item->nextItem() ) {
+                  // check if there is any item inside the now unavailable area
+                  // If so, we have to move _all_ items
+                  // If not, we don't have to move any item (avoids bug:117868)
+                  if ( (item->x() < wr.x()) || (item->y() < wr.y()) ) {
+                    needRepaint = true;
+                    break;
+                  }
+              }
+          else  // the iconArea was shifted left/up; more space now - use it
+              needRepaint = true;
+
+            if ( needRepaint )
+                for ( item = firstItem(); item; item = item->nextItem() )
+                    item->moveBy( dx, dy );
         }
+
         for ( item = firstItem(); item; item = item->nextItem() ) {
             QRect r( item->rect() );
             int dx = 0, dy = 0;
@@ -1377,6 +1483,7 @@ void KDIconView::updateWorkArea( const QRect &wr )
         if ( needRepaint ) {
             viewport()->repaint( FALSE );
             repaint( FALSE );
+            saveIconPositions();
         }
     }
 }
@@ -1542,9 +1649,7 @@ void KDIconView::saveIconPositions()
   QIconViewItem *it = firstItem();
   if ( !it )
     return; // No more icons. Maybe we're closing and they've been removed already
-  QRect desk = desktopRect();
-  QString X_w = QString( "X %1" ).arg( desk.width() );
-  QString Y_h = QString( "Y %1" ).arg( desk.height() );
+
   while ( it )
   {
     KFileIVI *ivi = static_cast<KFileIVI *>( it );
@@ -1552,7 +1657,7 @@ void KDIconView::saveIconPositions()
 
     m_dotDirectory->setGroup( prefix + item->url().fileName() );
     kdDebug(1214) << "KDIconView::saveIconPositions " << item->url().fileName() << " " << it->x() << " " << it->y() << endl;
-    saveIconPosition(m_dotDirectory, it->x(), it->y(), iconArea());
+    saveIconPosition(m_dotDirectory, it->x(), it->y());
 
     it = it->nextItem();
   }
@@ -1560,7 +1665,4 @@ void KDIconView::saveIconPositions()
   m_dotDirectory->sync();
 }
 
-#undef saveIconPosition
-#undef readIconPosition
-
 #include "kdiconview.moc"
