@@ -16,26 +16,12 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <qfile.h>
+#include <dcopclient.h>
 #include <assert.h>
 
 #include "lockeng.h"
 #include "lockeng.moc"
-
-#include <X11/Xlib.h>
-
-#ifdef HAVE_DPMS
-extern "C" {
-#include <X11/Xmd.h>
-#ifndef Bool
-#define Bool BOOL
-#endif
-#include <X11/extensions/dpms.h>
-
-#ifndef HAVE_DPMSINFO_PROTO
-Status DPMSInfo ( Display *, CARD16 *, BOOL * );
-#endif
-}
-#endif
+#include "kdesktopsettings.h"
 
 #include "xautolock_c.h"
 extern xautolock_corner_t xautolock_corners[ 4 ];
@@ -56,9 +42,6 @@ SaverEngine::SaverEngine()
     XGetScreenSaver(qt_xdisplay(), &mXTimeout, &mXInterval,
                     &mXBlanking, &mXExposures);
 
-    // We'll handle blanking
-    XSetScreenSaver(qt_xdisplay(), 0, mXInterval, mXBlanking, mXExposures);
-
     mState = Waiting;
     mXAutoLock = 0;
     mEnabled = false;
@@ -84,12 +67,50 @@ SaverEngine::~SaverEngine()
 }
 
 //---------------------------------------------------------------------------
+
+// This should be called only using DCOP.
 void SaverEngine::lock()
 {
+    bool ok = true;
     if (mState == Waiting)
     {
-        startLockProcess( ForceLock );
+        ok = startLockProcess( ForceLock );
+// It takes a while for kdesktop_lock to start and lock the screen.
+// Therefore delay the DCOP call until it tells kdesktop that the locking is in effect.
+// This is done only for --forcelock .
+        if( ok && mState != Saving )
+        {
+            DCOPClientTransaction* trans = kapp->dcopClient()->beginTransaction();
+            mLockTransactions.append( trans );
+        }
     }
+    else
+    {
+        mLockProcess.kill( SIGHUP );
+    }
+}
+
+void SaverEngine::processLockTransactions()
+{
+    for( QValueVector< DCOPClientTransaction* >::ConstIterator it = mLockTransactions.begin();
+         it != mLockTransactions.end();
+         ++it )
+    {
+        QCString replyType = "void";
+        QByteArray arr;
+        kapp->dcopClient()->endTransaction( *it, replyType, arr );
+    }
+    mLockTransactions.clear();
+}
+
+void SaverEngine::saverLockReady()
+{
+    if( mState != Preparing )
+    {
+	kdDebug( 1204 ) << "Got unexpected saverReady()" << endl;
+    }
+    kdDebug( 1204 ) << "Saver Lock Ready" << endl;
+    processLockTransactions();
 }
 
 //---------------------------------------------------------------------------
@@ -104,7 +125,7 @@ void SaverEngine::save()
 //---------------------------------------------------------------------------
 void SaverEngine::quit()
 {
-    if (mState == Saving)
+    if (mState == Saving || mState == Preparing)
     {
         stopLockProcess();
     }
@@ -135,8 +156,13 @@ bool SaverEngine::enable( bool e )
 	    connect(mXAutoLock, SIGNAL(timeout()), SLOT(idleTimeout()));
 	}
         mXAutoLock->setTimeout(mTimeout);
+        mXAutoLock->setDPMS(true);
 	//mXAutoLock->changeCornerLockStatus( mLockCornerTopLeft, mLockCornerTopRight, mLockCornerBottomLeft, mLockCornerBottomRight);
 
+        // We'll handle blanking
+        XSetScreenSaver(qt_xdisplay(), mTimeout + 10, mXInterval, PreferBlanking, mXExposures);
+        kdDebug() << "XSetScreenSaver " << mTimeout + 10 << endl;
+
         mXAutoLock->start();
 
         kdDebug(1204) << "Saver Engine started, timeout: " << mTimeout << endl;
@@ -149,6 +175,8 @@ bool SaverEngine::enable( bool e )
 	    mXAutoLock = 0;
 	}
 
+        XForceScreenSaver(qt_xdisplay(), ScreenSaverReset );
+        XSetScreenSaver(qt_xdisplay(), 0, mXInterval,  PreferBlanking, DontAllowExposures);
         kdDebug(1204) << "Saver Engine disabled" << endl;
     }
 
@@ -172,22 +200,22 @@ void SaverEngine::configure()
         return;
 
     // create a new config obj to ensure we read the latest options
-    KConfig *config = KGlobal::config();
-    config->reparseConfiguration();
+    KDesktopSettings::self()->readConfig();
 
-    config->setGroup("ScreenSaver");
-
-    bool e  = config->readBoolEntry("Enabled", false);
-    mTimeout = config->readNumEntry("Timeout", 300);
+    bool e  = KDesktopSettings::screenSaverEnabled();
+    mTimeout = KDesktopSettings::timeout();
 
     mEnabled = !e;   // force the enable()
-#ifdef HAVE_DPMS
-    mDPMS = config->readBoolEntry("DPMS-dependent", false);
-#endif
-    xautolock_corners[ 0 ] = config->readBoolEntry("LockCornerTopLeft", false) ? ca_forceLock :ca_nothing;
-    xautolock_corners[ 1 ] = config->readBoolEntry("LockCornerTopRight", false) ? ca_forceLock :ca_nothing;
-    xautolock_corners[ 2 ] = config->readBoolEntry("LockCornerBottomLeft", false) ? ca_forceLock :ca_nothing;
-    xautolock_corners[ 3 ] = config->readBoolEntry("LockCornerBottomRight", false) ? ca_forceLock :ca_nothing;
+
+    int action;
+    action = KDesktopSettings::actionTopLeft();
+    xautolock_corners[0] = applyManualSettings(action);
+    action = KDesktopSettings::actionTopRight();
+    xautolock_corners[1] = applyManualSettings(action);
+    action = KDesktopSettings::actionBottomLeft();
+    xautolock_corners[2] = applyManualSettings(action);
+    action = KDesktopSettings::actionBottomRight();
+    xautolock_corners[3] = applyManualSettings(action);
 
     enable( e );
 }
@@ -207,13 +235,10 @@ void SaverEngine::setBlankOnly( bool blankOnly )
 //
 // Start the screen saver.
 //
-void SaverEngine::startLockProcess( LockType lock_type )
+bool SaverEngine::startLockProcess( LockType lock_type )
 {
     if (mState != Waiting)
-    {
-        kdWarning(1204) << "SaverEngine::startSaver() saver already active" << endl;
-        return;
-    }
+        return true;
 
     kdDebug(1204) << "SaverEngine: starting saver" << endl;
     emitDCOPSignal("KDE_start_screensaver()", QByteArray());
@@ -227,7 +252,7 @@ void SaverEngine::startLockProcess( LockType lock_type )
     if( path.isEmpty())
     {
 	kdDebug( 1204 ) << "Can't find kdesktop_lock!" << endl;
-	return;
+	return false;
     }
     mLockProcess << path;
     switch( lock_type )
@@ -247,14 +272,16 @@ void SaverEngine::startLockProcess( LockType lock_type )
     if (mLockProcess.start() == false )
     {
 	kdDebug( 1204 ) << "Failed to start kdesktop_lock!" << endl;
-	return;
+	return false;
     }
+    XSetScreenSaver(qt_xdisplay(), 0, mXInterval,  PreferBlanking, mXExposures);
 
-    mState = Saving;
+    mState = Preparing;
     if (mXAutoLock)
     {
         mXAutoLock->stop();
     }
+    return true;
 }
 
 //---------------------------------------------------------------------------
@@ -271,13 +298,18 @@ void SaverEngine::stopLockProcess()
     kdDebug(1204) << "SaverEngine: stopping lock" << endl;
     emitDCOPSignal("KDE_stop_screensaver()", QByteArray());
 
-
     mLockProcess.kill();
 
-    if (mXAutoLock)
+    if (mEnabled)
     {
-        mXAutoLock->start();
+        if (mXAutoLock)
+        {
+            mXAutoLock->start();
+        }
+        XForceScreenSaver(qt_xdisplay(), ScreenSaverReset );
+        XSetScreenSaver(qt_xdisplay(), mTimeout + 10, mXInterval, PreferBlanking, mXExposures);
     }
+    processLockTransactions();
     mState = Waiting;
 }
 
@@ -287,10 +319,16 @@ void SaverEngine::lockProcessExited()
     if( mState == Waiting )
 	return;
     emitDCOPSignal("KDE_stop_screensaver()", QByteArray());
-    if (mXAutoLock)
+    if (mEnabled)
     {
-        mXAutoLock->start();
+        if (mXAutoLock)
+        {
+            mXAutoLock->start();
+        }
+        XForceScreenSaver(qt_xdisplay(), ScreenSaverReset );
+        XSetScreenSaver(qt_xdisplay(), mTimeout + 10, mXInterval, PreferBlanking, mXExposures);
     }
+    processLockTransactions();
     mState = Waiting;
 }
 
@@ -300,19 +338,34 @@ void SaverEngine::lockProcessExited()
 //
 void SaverEngine::idleTimeout()
 {
-#ifdef HAVE_DPMS
-    if (mDPMS) {
-        BOOL on;
-        CARD16 state;
-        DPMSInfo( qt_xdisplay(), &state, &on );
-        if (!on) {
-            kdDebug(1204) << "Skip enabling screen saver because DPMS is off" << endl;
-            mXAutoLock->stop();
-            mXAutoLock->start();
-            return;
-        }
-    }
-#endif
-
+    // disable X screensaver
+    XForceScreenSaver(qt_xdisplay(), ScreenSaverReset );
+    XSetScreenSaver(qt_xdisplay(), 0, mXInterval, PreferBlanking, DontAllowExposures);
     startLockProcess( DefaultLock );
 }
+
+xautolock_corner_t SaverEngine::applyManualSettings(int action)
+{
+	if (action == 0)
+	{
+		kdDebug() << "no lock" << endl;
+		return ca_nothing;
+	}
+	else
+	if (action == 1)
+	{
+		kdDebug() << "lock screen" << endl;
+		return ca_forceLock;
+	}
+	else
+	if (action == 2)
+	{
+		kdDebug() << "prevent lock" << endl;
+		return ca_dontLock;
+	}
+	else
+	{
+		kdDebug() << "no lock nothing" << endl;
+		return ca_nothing;
+	}
+}
