@@ -4,32 +4,48 @@
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
  */
 
+#include <config.h>
 #include <qstring.h>
+#include <kconfig.h>
+#include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 
 #include <kdesu/su.h>
 #include "sudlg.h"
 
-
-KDEsuDialog::KDEsuDialog(QCString user, QCString auth_user, bool enableKeep, const QString& icon)
-    : KPasswordDialog(Password, enableKeep, User1, icon)
+KDEsuDialog::KDEsuDialog(QCString user, QCString auth_user, bool enableKeep,const QString& icon, bool withIgnoreButton)
+     : KPasswordDialog(Password, enableKeep, (withIgnoreButton ? User1:NoDefault), icon)
 {
+    KConfig* config = KGlobal::config();
+    config->setGroup("super-user-command");
+    QString superUserCommand = config->readEntry("super-user-command", DEFAULT_SUPER_USER_COMMAND);
+    if ( superUserCommand != "sudo" && superUserCommand != "su" ) {
+	kdWarning() << "unknown super user command" << endl;
+	superUserCommand = "su";
+    }
+
     m_User = auth_user;
     setCaption(i18n("Run as %1").arg(user));
 
     QString prompt;
-    if (m_User == "root")
-	prompt = i18n("The action you requested needs root privileges. "
-		"Please enter root's password below or click "
-		"Ignore to continue with your current privileges.");
-    else
-	prompt = i18n("The action you requested needs additional privileges. "
+    if (superUserCommand == "sudo" && m_User == "root") {
+	    prompt = i18n("Please enter your password." );
+    } else {
+        if (m_User == "root") {
+	    prompt = i18n("The action you requested needs root privileges. "
+	    "Please enter root's password below or click "
+	    "Ignore to continue with your current privileges.");
+        } else {
+	    prompt = i18n("The action you requested needs additional privileges. "
 		"Please enter the password for \"%1\" below or click "
 		"Ignore to continue with your current privileges.").arg(m_User);
+	}
+    }
     setPrompt(prompt);
 
-    setButtonText(User1, i18n("&Ignore"));
+    if( withIgnoreButton )
+	setButtonText(User1, i18n("&Ignore"));
 }
 
 
