@@ -19,6 +19,7 @@
 #include <sys/socket.h>
 
 #include <kdebug.h>
+#include <ksockaddr.h>
 #include "secure.h"
 
 
@@ -30,7 +31,7 @@
 
 SocketSecurity::SocketSecurity(int sockfd)
 {
-    ksize_t len = sizeof(struct ucred);
+    ksocklen_t len = sizeof(struct ucred);
     if (getsockopt(sockfd, SOL_SOCKET, SO_PEERCRED, &cred, &len) < 0) {
 	kdError() << "getsockopt(SO_PEERCRED) " << perror << endl;
 	return;
