@@ -26,6 +26,7 @@
 #
 
 use strict;
+use Cwd 'abs_path';
 
 sub pegout($)
 {
@@ -33,6 +34,18 @@ sub pegout($)
   exit 1;
 }
 
+sub relpath($$)
+{
+  my @src = split(/\//, abs_path(shift));
+  my @dst = split(/\//, abs_path(shift));
+  pop @dst;
+  while (@src && @dst && $src[0] eq $dst[0]) {
+    shift @src;
+    shift @dst;
+  }
+  return "../"x@dst . join("/", @src);
+}
+
 sub getl()
 {
   while (<INFILE>) {
@@ -670,12 +683,15 @@ while (<INFILE>) {
 emit_section();
 close INFILE;
 
+my $srcf = relpath($ARGV[0], $ARGV[1]);
+my $exen = relpath($0, $ARGV[1]);
+
 open (OUTFILE, ">".$ARGV[1]) || pegout("$0: cannot create output file ".$ARGV[1]);
 
 if (!$do_doc) {
 
 print OUTFILE
-  "/* generated from $ARGV[0] by $0 - DO NOT EDIT! */\n\n".
+  "/* generated from $srcf by $exen - DO NOT EDIT! */\n\n".
   "#ifndef CONFIG_DEFS\n".
   "#define CONFIG_DEFS\n\n".
   $raw_out."\n\n".
@@ -814,7 +830,7 @@ print OUTFILE
 
 $doc =~ s/%REF%/$doc_ref/;
 print OUTFILE
-  "<!-- generated from $ARGV[0] - DO NOT EDIT! -->\n\n".
+  "<!-- generated from $srcf - DO NOT EDIT! -->\n\n".
   $doc;
 
 }
