@@ -26,7 +26,7 @@
 KfindDlg::KfindDlg(const KURL & url, QWidget *parent, const char *name)
   : KDialogBase( Plain, QString::null,
 	User1 | User2 | Apply | Close | Help, Apply,
-        parent, name, true, true,
+        parent, name, true, false,
 	KGuiItem(i18n("Stop"), "stop"),
 	KStdGuiItem::saveAs())
 {
