@@ -28,6 +28,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <qintdict.h>
 
 #include <kpanelapplet.h>
+#include <kwin.h>
 
 #include "pagerbutton.h"
 #include "pagersettings.h"
@@ -39,10 +40,7 @@ class QTimer;
 class KProcess;
 class KWinModule;
 class KShadowEngine;
-
-namespace KWin { 
-    class WindowInfo;
-}
+class KSelectionOwner;
 
 class PagerSettings;
 
@@ -127,6 +125,7 @@ private:
     int desktopLayoutOrientation;
     int desktopLayoutX;
     int desktopLayoutY;
+    KSelectionOwner* m_desktopLayoutOwner;
 
     KWinModule *m_kwin;
     KShadowEngine* m_shadowEngine;
