@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "NaughtyApplet.h"
@@ -38,8 +38,7 @@
 
 extern "C"
 {
-    KPanelApplet *
-  init(QWidget * parent, const QString & configFile)
+  KDE_EXPORT KPanelApplet*  init(QWidget * parent, const QString & configFile)
   {
     KGlobal::locale()->insertCatalogue("naughtyapplet");
 
@@ -115,7 +114,7 @@ NaughtyApplet::slotWarn(ulong pid, const QString & name)
                    "this, or it may just be busy.\n"
                    "Would you like to try to stop the program?");
 
-  int retval = KMessageBox::warningYesNo(this, s.arg(name));
+  int retval = KMessageBox::warningYesNo(this, s.arg(name), QString::null, i18n("Stop"), i18n("Keep Running"));
 
   if (KMessageBox::Yes == retval)
     monitor_->kill(pid);
@@ -123,7 +122,7 @@ NaughtyApplet::slotWarn(ulong pid, const QString & name)
   {
     s = i18n("In future, should busy programs called '%1' be ignored?");
 
-    retval = KMessageBox::questionYesNo(this, s.arg(name));
+    retval = KMessageBox::questionYesNo(this, s.arg(name), QString::null, i18n("Ignore"), i18n("Do Not Ignore"));
 
     if (KMessageBox::Yes == retval)
     {
