@@ -31,7 +31,7 @@
 
 extern "C"
 {
-   KPanelExtension *init( QWidget *parent, const QString& configFile )
+   KDE_EXPORT KPanelExtension *init( QWidget *parent, const QString& configFile )
    {
       KGlobal::locale()->insertCatalogue("kickersidebarextension");
       KGlobal::locale()->insertCatalogue("konqueror");
@@ -75,7 +75,7 @@ SidebarExtension::SidebarExtension( const QString& configFile,
     m_resizeHandle->setFixedWidth(6);
     m_resizeHandle->setCursor(QCursor(Qt::SizeHorCursor));
     connect(p->widget(),SIGNAL(panelHasBeenExpanded(bool)),this,SLOT(needLayoutUpdate(bool)));
-    needLayoutUpdate(true);
+    needLayoutUpdate(false);
     m_resizeHandle->installEventFilter(this);
     m_resizeHandle->setMouseTracking(true);
 //   l->add(p->widget());
@@ -85,15 +85,17 @@ SidebarExtension::SidebarExtension( const QString& configFile,
 }
 
 void SidebarExtension::needLayoutUpdate(bool exp) {
+	setReserveStrut(!exp); // only reserve a strut when we are collapsed
 	if (exp) {
 		m_currentWidth=m_expandedSize;
 		m_resizeHandle->show();
+		raise();
 	} else {
 		m_currentWidth=24;
 		m_resizeHandle->hide();
 	}
 	topLevelWidget()->setFixedWidth(m_currentWidth);
-	updateLayout();
+	emit updateLayout();
 }
 
 void SidebarExtension::openURLRequest( const KURL &url, const KParts::URLArgs &) {
