@@ -67,7 +67,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ExtensionContainer::ExtensionContainer(const AppletInfo& info,
                                        const QString& extensionId,
                                        QWidget *parent)
-  : QFrame(parent, "ExtensionContainer", WStyle_Customize | WStyle_NoBorder),
+  : QFrame(parent, ("ExtensionContainer#" + extensionId).latin1(), WStyle_Customize | WStyle_NoBorder),
     m_settings(KSharedConfig::openConfig(info.configFile())),
     m_hideMode(ManualHide),
     m_unhideTriggeredAt(UnhideTrigger::None),
@@ -94,7 +94,7 @@ ExtensionContainer::ExtensionContainer(KPanelExtension* extension,
                                        const AppletInfo& info,
                                        const QString& extensionId,
                                        QWidget *parent)
-  : QFrame(parent, "ExtensionContainer", WStyle_Customize | WStyle_NoBorder),
+  : QFrame(parent, ("ExtensionContainer#" + extensionId).latin1(), WStyle_Customize | WStyle_NoBorder),
     m_settings(KSharedConfig::openConfig(info.configFile())),
     _autoHidden(false),
     _userHidden(Unhidden),
@@ -142,11 +142,11 @@ void ExtensionContainer::init()
     _layout->setColStretch(1,10);
 
     // instantiate the autohide timer
-    _autohideTimer = new QTimer(this);
+    _autohideTimer = new QTimer(this, "_autohideTimer");
     connect(_autohideTimer, SIGNAL(timeout()), SLOT(autoHideTimeout()));
 
     // instantiate the updateLayout event compressor timer
-    _updateLayoutTimer = new QTimer(this);
+    _updateLayoutTimer = new QTimer(this, "_updateLayoutTimer");
     connect(_updateLayoutTimer, SIGNAL(timeout()), SLOT(actuallyUpdateLayout()));
 
     installEventFilter(this); // for mouse event handling
@@ -190,6 +190,7 @@ void ExtensionContainer::init()
         connect(m_extension, SIGNAL(updateLayout()), SLOT(updateLayout()));
         connect(m_extension, SIGNAL(maintainFocus(bool)),
                 SLOT(maintainFocus(bool)));
+
         _layout->addWidget(m_extension, 1, 1);
     }
 
@@ -541,8 +542,6 @@ void ExtensionContainer::actuallyUpdateLayout()
 //    kdDebug(1210) << "PanelContainer::updateLayout()" << endl;
     resetLayout();
     updateWindowManager();
-
-    emit layoutUpdated(this);
 }
 
 void ExtensionContainer::enableMouseOverEffects()
