@@ -462,12 +462,9 @@ void PanelKMenu::setMaximumSize(int w, int h)
   KPanelMenu::setMaximumSize(w + sidePixmap.width(), h);
 }
 
-#include <iostream>
-using namespace std;
-
 void PanelKMenu::showMenu()
 {
-    cout << "PanelKMenu::showMenu()" << endl;
+    kdDebug( 1210 ) << "PanelKMenu::showMenu()" << endl;
     PanelPopupButton *kButton = MenuManager::the()->findKButtonFor(this);
     if (kButton)
     {
