@@ -702,6 +702,12 @@ void TaskBar::reLayout()
         // actual button height
         int bheight = contentsRect().height() / rows;
 
+        // avoid zero devision later
+        if (bheight < 1)
+        {
+            bheight = 1;
+        }
+
         // buttons per row
         int bpr = (int)ceil( (double)list.count() / rows);
 
