@@ -34,6 +34,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 // #include <kpixmapio.h>
 #include <kstaticdeleter.h>
 #include <kwinmodule.h>
+#include <kxerrorhandler.h>
 #include <netwm.h>
 
 #include "taskmanager.h"
@@ -1444,7 +1445,10 @@ void Task::updateWindowPixmap()
         XFreePixmap(dpy, m_windowPixmap);
     }
 
+    KXErrorHandler err;
     m_windowPixmap = XCompositeNameWindowPixmap(dpy, m_frameId);
+    if( err.error( true ))
+        m_windowPixmap = None;
 #endif // THUMBNAILING_POSSIBLE
 }
 
