@@ -18,6 +18,7 @@
 */
 
 #include "notificationdialog.h"
+#include <qlayout.h>
 
 #include <krun.h>
 #include <klocale.h>
@@ -39,8 +40,13 @@ NotificationDialog::NotificationDialog( KFileItem medium, NotifierSettings *sett
 	setCaption( KIO::decodeFileName(m_medium.name()) );
 	clearWState( WState_Polished );
 
-	m_view = new NotificationDialogView( this );
+	QWidget *page = new QWidget( this );
+	setMainWidget(page);
+	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 
+	m_view = new NotificationDialogView( page );
+
+	topLayout->addWidget(m_view);
 	m_view->iconLabel->setPixmap( m_medium.pixmap(64) );
 	m_view->mimetypeLabel->setText( i18n( "<b>Medium type:</b>" ) + " "
 	                              + m_medium.mimeTypePtr()->comment() );
@@ -49,7 +55,6 @@ NotificationDialog::NotificationDialog( KFileItem medium, NotifierSettings *sett
 
 	resize( QSize(400,400).expandedTo( minimumSizeHint() ) );
 
-	setMainWidget( m_view );
 
 	m_actionWatcher = new KDirWatch();
 	QString services_dir
