@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "KonquerorIface.h"
@@ -29,6 +29,7 @@
 #include <dcopclient.h>
 #include <kdebug.h>
 #include <qfile.h>
+#include "konq_settingsxt.h"
 
 // these DCOP calls come from outside, so any windows created by these
 // calls would have old user timestamps (for KWin's no-focus-stealing),
@@ -66,25 +67,19 @@ DCOPRef KonquerorIface::openBrowserWindowASN( const QString &url, const QCString
 
 DCOPRef KonquerorIface::createNewWindow( const QString &url )
 {
-    qt_x_user_time = 0;
-    // Filter the URL, so that "kfmclient openURL gg:foo" works also when konq is already running
-    KURL finalURL = KonqMisc::konqFilteredURL( 0, url );
-    KonqMainWindow *res = KonqMisc::createNewWindow( finalURL );
-    if ( !res )
-        return DCOPRef();
-    return res->dcopObject();
+    return createNewWindow( url, QString::null, false );
 }
 
-DCOPRef KonquerorIface::createNewWindowASN( const QString &url, const QCString& startup_id )
+DCOPRef KonquerorIface::createNewWindowASN( const QString &url, const QCString& startup_id, bool tempFile )
 {
     kapp->setStartupId( startup_id );
-    return createNewWindow( url );
+    return createNewWindow( url, QString::null, tempFile );
 }
 
 DCOPRef KonquerorIface::createNewWindowWithSelection( const QString &url, QStringList filesToSelect )
 {
     qt_x_user_time = 0;
-    KonqMainWindow *res = KonqMisc::createNewWindow( KURL(url), KParts::URLArgs(), false, filesToSelect  );
+    KonqMainWindow *res = KonqMisc::createNewWindow( KURL(url), KParts::URLArgs(), false, filesToSelect );
     if ( !res )
         return DCOPRef();
     return res->dcopObject();
@@ -96,24 +91,24 @@ DCOPRef KonquerorIface::createNewWindowWithSelectionASN( const QString &url, QSt
     return createNewWindowWithSelection( url, filesToSelect );
 }
 
-DCOPRef KonquerorIface::createNewWindow( const QString &url, const QString &mimetype )
+DCOPRef KonquerorIface::createNewWindow( const QString &url, const QString &mimetype, bool tempFile )
 {
     qt_x_user_time = 0;
     KParts::URLArgs args;
     args.serviceType = mimetype;
     // Filter the URL, so that "kfmclient openURL gg:foo" works also when konq is already running
     KURL finalURL = KonqMisc::konqFilteredURL( 0, url );
-    KonqMainWindow *res = KonqMisc::createNewWindow( finalURL, args );
+    KonqMainWindow *res = KonqMisc::createNewWindow( finalURL, args, false, QStringList(), tempFile );
     if ( !res )
         return DCOPRef();
     return res->dcopObject();
 }
 
 DCOPRef KonquerorIface::createNewWindowASN( const QString &url, const QString &mimetype,
-    const QCString& startup_id )
+    const QCString& startup_id, bool tempFile )
 {
     kapp->setStartupId( startup_id );
-    return createNewWindow( url, mimetype );
+    return createNewWindow( url, mimetype, tempFile );
 }
 
 DCOPRef KonquerorIface::createBrowserWindowFromProfile( const QString &path )
@@ -254,21 +249,21 @@ bool KonquerorIface::processCanBeReused( int screen )
     QPtrList<KonqMainWindow>* windows = KonqMainWindow::mainWindowList();
     if( windows == NULL )
         return true;
-    KConfig* cfg = kapp->config();
-    KConfigGroupSaver saver( cfg, "Reusing" );
-    QStringList allowed_parts;
-    // is duplicated in client/kfmclient.cc
-    allowed_parts << QString::fromLatin1( "konq_iconview.desktop" )
-                  << QString::fromLatin1( "konq_multicolumnview.desktop" )
-                  << QString::fromLatin1( "konq_sidebartng.desktop" )
-                  << QString::fromLatin1( "konq_infolistview.desktop" )
-                  << QString::fromLatin1( "konq_treeview.desktop" )
-                  << QString::fromLatin1( "konq_detailedlistview.desktop" );
+    QStringList allowed_parts = KonqSettings::safeParts();
     bool all_parts_allowed = false;
-    if( cfg->hasKey( "SafeParts" )
-        && cfg->readEntry( "SafeParts" ) != QString::fromLatin1( "SAFE" ))
-        allowed_parts = cfg->readListEntry( "SafeParts" );
-    if( allowed_parts.count() == 1 && allowed_parts.first() == QString::fromLatin1( "ALL" ))
+    
+    if( allowed_parts.count() == 1 && allowed_parts.first() == QString::fromLatin1( "SAFE" ))
+    {
+        allowed_parts.clear();
+        // is duplicated in client/kfmclient.cc
+        allowed_parts << QString::fromLatin1( "konq_iconview.desktop" )
+                      << QString::fromLatin1( "konq_multicolumnview.desktop" )
+                      << QString::fromLatin1( "konq_sidebartng.desktop" )
+                      << QString::fromLatin1( "konq_infolistview.desktop" )
+                      << QString::fromLatin1( "konq_treeview.desktop" )
+                      << QString::fromLatin1( "konq_detailedlistview.desktop" );
+    }
+    else if( allowed_parts.count() == 1 && allowed_parts.first() == QString::fromLatin1( "ALL" ))
     {
         allowed_parts.clear();
         all_parts_allowed = true;
