@@ -56,7 +56,7 @@ static const char description[] = I18N_NOOP("KDE tool for opening URLs from the
 static const char version[] = "2.0";
 
 QCString clientApp::startup_id_str;
-bool clientApp::m_ok;
+bool clientApp::m_ok = true;
 bool s_interactive = true;
 
 static const KCmdLineOptions options[] =
@@ -275,6 +275,24 @@ static QCString konqyToReuse( const QString& url, const QString& mimetype, const
     return ret;
 }
 
+static bool krun_has_error = false;
+
+void clientApp::sendASNChange()
+{
+    KStartupInfoId id;
+    id.initId( startup_id_str );
+    KStartupInfoData data;
+    data.addPid( 0 );   // say there's another process for this ASN with unknown PID
+    data.setHostname(); // ( no need to bother to get this konqy's PID )
+    Display* dpy = qt_xdisplay();
+    if( dpy == NULL ) // we may be running without QApplication here
+        dpy = XOpenDisplay( NULL );
+    if( dpy != NULL )
+        KStartupInfo::sendChangeX( dpy, id, data );
+    if( dpy != NULL && dpy != qt_xdisplay())
+        XCloseDisplay( dpy );
+}
+
 bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, const QString & mimetype)
 {
     kdDebug( 1202 ) << "clientApp::createNewWindow " << url.url() << " mimetype=" << mimetype << endl;
@@ -302,11 +320,11 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, co
             clientApp app;
             KStartupInfo::appStarted();
 
-            KRun * run = new KRun( url ); // TODO pass tempFile [needs support in the KRun ctor]
+            KRun * run = new KRun( url, 0, 0, false, false /* no progress window */ ); // TODO pass tempFile [needs support in the KRun ctor]
             QObject::connect( run, SIGNAL( finished() ), &app, SLOT( delayedQuit() ));
             QObject::connect( run, SIGNAL( error() ), &app, SLOT( delayedQuit() ));
             app.exec();
-            return m_ok;
+            return !krun_has_error;
         }
     }
 
@@ -321,9 +339,9 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, co
             "windowCanBeUsedForTab()", data, foundApp, foundObj, false, 3000 ) )
         {
             DCOPRef ref( foundApp, foundObj );
-            DCOPReply reply = ref.call( "newTab", url.url(), tempFile );
+            DCOPReply reply = ref.call( "newTabASN", url.url(), startup_id_str, tempFile );
             if ( reply.isValid() ) {
-                KStartupInfo::appStarted();
+                sendASNChange();
                 return true;
             }
       }
@@ -335,18 +353,7 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, co
         kdDebug( 1202 ) << "clientApp::createNewWindow using existing konqueror" << endl;
         KonquerorIface_stub konqy( appId, "KonquerorIface" );
         konqy.createNewWindowASN( url.url(), mimetype, startup_id_str, tempFile );
-        KStartupInfoId id;
-        id.initId( startup_id_str );
-        KStartupInfoData data;
-        data.addPid( 0 );   // say there's another process for this ASN with unknown PID
-        data.setHostname(); // ( no need to bother to get this konqy's PID )
-        Display* dpy = qt_xdisplay();
-        if( dpy == NULL ) // we may be running without QApplication here
-            dpy = XOpenDisplay( NULL );
-        if( dpy != NULL )
-            KStartupInfo::sendChangeX( dpy, id, data );
-        if( dpy != NULL && dpy != qt_xdisplay())
-            XCloseDisplay( dpy );
+        sendASNChange();
     }
     else
     {
@@ -406,18 +413,7 @@ bool clientApp::openProfile( const QString & profileName, const QString & url, c
   else
       konqy.createBrowserWindowFromProfileAndURLASN( profile, profileName, url, mimetype, startup_id_str );
   sleep(2); // Martin Schenk <martin@schenk.com> says this is necessary to let the server read from the socket
-  KStartupInfoId id;
-  id.initId( startup_id_str );
-  KStartupInfoData sidata;
-  sidata.addPid( 0 );   // say there's another process for this ASN with unknown PID
-  sidata.setHostname(); // ( no need to bother to get this konqy's PID )
-  Display* dpy = qt_xdisplay();
-  if( dpy == NULL ) // we may be running without QApplication here
-      dpy = XOpenDisplay( NULL );
-  if( dpy != NULL )
-      KStartupInfo::sendChangeX( dpy, id, sidata );
-  if( dpy != NULL && dpy != qt_xdisplay())
-      XCloseDisplay( dpy );
+  sendASNChange();
   return true;
 }
 
@@ -426,6 +422,9 @@ void clientApp::delayedQuit()
     // Quit in 2 seconds. This leaves time for KRun to pop up
     // "app not found" in KProcessRunner, if that was the case.
     QTimer::singleShot( 2000, this, SLOT(deref()) );
+    // don't access the KRun instance later, it will be deleted after calling slots
+    if( static_cast< const KRun* >( sender())->hasError())
+        krun_has_error = true;
 }
 
 static void checkArgumentCount(int count, int min, int max)
@@ -504,6 +503,7 @@ bool clientApp::doIt()
     checkArgumentCount(argc, 2, 2);
     KPropertiesDialog * p = new KPropertiesDialog( args->url(1) );
     QObject::connect( p, SIGNAL( destroyed() ), &app, SLOT( quit() ));
+    QObject::connect( p, SIGNAL( canceled() ), &app, SLOT( slotDialogCanceled() ));
     app.exec();
     return m_ok;
   }
@@ -517,11 +517,11 @@ bool clientApp::doIt()
     }
     else if ( argc == 2 )
     {
-      KRun * run = new KRun( args->url(1) );
+      KRun * run = new KRun( args->url(1), 0, 0, false, false /* no progress window */ );
       QObject::connect( run, SIGNAL( finished() ), &app, SLOT( delayedQuit() ));
       QObject::connect( run, SIGNAL( error() ), &app, SLOT( delayedQuit() ));
       app.exec();
-      return m_ok;
+      return !krun_has_error;
     }
     else if ( argc == 3 )
     {
@@ -634,4 +634,10 @@ void clientApp::slotResult( KIO::Job * job )
   quit();
 }
 
+void clientApp::slotDialogCanceled()
+{
+    m_ok = false;
+    quit();
+}
+
 #include "kfmclient.moc"
