@@ -1,4 +1,4 @@
-// -*- mode:cperl; cperl-indent-level:4; cperl-continued-statement-offset:4; indent-tabs-mode:nil -*-
+// -*- indent-tabs-mode:nil -*-
 // vim: set ts=4 sts=4 sw=4 et:
 /* This file is part of the KDE project
    Copyright (C) 2000 David Faure <faure@kde.org>
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "actionsimpl.h"
@@ -28,6 +28,7 @@
 #include "testlink.h"
 #include "listview.h"
 #include "exporters.h"
+#include "bookmarkinfo.h"
 
 #include <stdlib.h>
 
@@ -52,7 +53,7 @@
 
 #include <kdatastream.h>
 #include <ktempfile.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 
 #include <kparts/part.h>
 #include <kparts/componentfactory.h>
@@ -64,15 +65,14 @@
 #include <kbookmarkmanager.h>
 #include <kbookmarkimporter.h>
 
-#include <kbookmarkimporter.h>
 #include <kbookmarkimporter_ie.h>
 #include <kbookmarkimporter_opera.h>
 #include <kbookmarkexporter.h>
 
 ActionsImpl* ActionsImpl::s_self = 0;
 
-// decoupled from resetActions in toplevel.cpp 
-// as resetActions simply uses the action groups 
+// decoupled from resetActions in toplevel.cpp
+// as resetActions simply uses the action groups
 // specified in the ui.rc file
 void KEBApp::createActions() {
 
@@ -81,13 +81,10 @@ void KEBApp::createActions() {
     // save and quit should probably not be in the toplevel???
     (void) KStdAction::quit(
         this, SLOT( close() ), actionCollection());
-    (void) KStdAction::keyBindings(
-        this, SLOT( slotConfigureKeyBindings() ), actionCollection());
+    KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()), actionCollection());
     (void) KStdAction::configureToolbars(
         this, SLOT( slotConfigureToolbars() ), actionCollection());
 
-    (void) KStdAction::save(
-        actn, SLOT( slotSave() ), actionCollection());
     if (m_browser) {
         (void) KStdAction::open(
             actn, SLOT( slotLoad() ), actionCollection());
@@ -102,26 +99,8 @@ void KEBApp::createActions() {
 
     // settings menu
     (void) new KToggleAction(
-        i18n("&Auto-Save on Program Close"), 0,
-        this, SLOT( slotSaveOnClose() ), actionCollection(), 
-        "settings_saveonclose");
-    (void) new KToggleAction(
-        i18n("Advanced Add Bookmark in Konqueror"), 0,
-        this, SLOT( slotAdvancedAddBookmark() ), actionCollection(), 
-        "settings_advancedaddbookmark");
-
-    // these options should be in a kcontrol thing
-    (void) new KToggleAction(
-        i18n("Display Only Marked Bookmarks in Konqueror Bookmark Toolbar"), 
-        0, this, SLOT( slotFilteredToolbar() ), actionCollection(), 
-        "settings_filteredtoolbar");
-    /* (void) new KToggleAction(
-            i18n("Split View (Very Experimental!)"), 0,
-            this, SLOT( slotSplitView() ), actionCollection(), 
-            "settings_splitview"); */
-    (void) new KToggleAction(
-        i18n("&Show Netscape Bookmarks in Konqueror Windows"), 0,
-        actn, SLOT( slotShowNS() ), actionCollection(), 
+        i18n("&Show Netscape Bookmarks in Konqueror"), 0,
+        actn, SLOT( slotShowNS() ), actionCollection(),
         "settings_showNS");
 
     // actions
@@ -138,7 +117,7 @@ void KEBApp::createActions() {
         i18n("C&hange Comment"), "text", Key_F4,
         actn, SLOT( slotChangeComment() ), actionCollection(), "changecomment");
     (void) new KAction(
-        i18n("Chan&ge Icon..."), 0,
+        i18n("Chan&ge Icon..."), "icons", 0,
         actn, SLOT( slotChangeIcon() ), actionCollection(), "changeicon");
     (void) new KAction(
         i18n("Update Favicon"), 0,
@@ -147,14 +126,14 @@ void KEBApp::createActions() {
         i18n("Recursive Sort"), 0,
         actn, SLOT( slotRecursiveSort() ), actionCollection(), "recursivesort");
     (void) new KAction(
-        i18n("&Create New Folder..."), "folder_new", CTRL+Key_N,
+        i18n("&New Folder..."), "folder_new", CTRL+Key_N,
         actn, SLOT( slotNewFolder() ), actionCollection(), "newfolder");
     (void) new KAction(
-        i18n("Create &New Bookmark"), "www", 0,
+        i18n("&New Bookmark"), "www", 0,
         actn, SLOT( slotNewBookmark() ), actionCollection(), "newbookmark");
     (void) new KAction(
         i18n("&Insert Separator"), CTRL+Key_I,
-        actn, SLOT( slotInsertSeparator() ), actionCollection(), 
+        actn, SLOT( slotInsertSeparator() ), actionCollection(),
         "insertseparator");
     (void) new KAction(
         i18n("&Sort Alphabetically"), 0,
@@ -165,6 +144,9 @@ void KEBApp::createActions() {
     (void) new KAction(
         i18n("Show in T&oolbar"), "bookmark_toolbar", 0,
         actn, SLOT( slotShowInToolbar() ), actionCollection(), "showintoolbar");
+    (void) new KAction(
+        i18n("Hide in T&oolbar"), "bookmark_toolbar", 0,
+        actn, SLOT( slotHideInToolbar() ), actionCollection(), "hideintoolbar");
     (void) new KAction(
         i18n("&Expand All Folders"), 0,
         actn, SLOT( slotExpandAll() ), actionCollection(), "expandall");
@@ -183,14 +165,14 @@ void KEBApp::createActions() {
         actn, SLOT( slotTestAll() ), actionCollection(), "testall" );
     (void) new KAction(
         i18n("Update All &Favicons"), 0,
-        actn, SLOT( slotUpdateAllFavIcons() ), actionCollection(), 
+        actn, SLOT( slotUpdateAllFavIcons() ), actionCollection(),
         "updateallfavicons" );
     (void) new KAction(
         i18n("Cancel &Checks"), 0,
         actn, SLOT( slotCancelAllTests() ), actionCollection(), "canceltests" );
     (void) new KAction(
         i18n("Cancel &Favicon Updates"), 0,
-        actn, SLOT( slotCancelFavIconUpdates() ), actionCollection(), 
+        actn, SLOT( slotCancelFavIconUpdates() ), actionCollection(),
         "cancelfaviconupdates" );
     (void) new KAction(
         i18n("Import &Netscape Bookmarks..."), "netscape", 0,
@@ -199,71 +181,39 @@ void KEBApp::createActions() {
         i18n("Import &Opera Bookmarks..."), "opera", 0,
         actn, SLOT( slotImport() ), actionCollection(), "importOpera");
     (void) new KAction(
-        i18n("Import All &Crash Sessions as Bookmarks..."), "opera", 0,
+        i18n("Import All &Crash Sessions as Bookmarks..."), 0,
         actn, SLOT( slotImport() ), actionCollection(), "importCrashes");
     (void) new KAction(
         i18n("Import &Galeon Bookmarks..."), 0,
         actn, SLOT( slotImport() ), actionCollection(), "importGaleon");
     (void) new KAction(
-        i18n("Import &KDE2 Bookmarks..."), 0,
+        i18n("Import &KDE2/KDE3 Bookmarks..."), 0,
         actn, SLOT( slotImport() ), actionCollection(), "importKDE2");
     (void) new KAction(
-        i18n("&Import IE Bookmarks..."), 0,
+        i18n("Import &IE Bookmarks..."), 0,
         actn, SLOT( slotImport() ), actionCollection(), "importIE");
     (void) new KAction(
         i18n("Import &Mozilla Bookmarks..."), "mozilla", 0,
         actn, SLOT( slotImport() ), actionCollection(), "importMoz");
     (void) new KAction(
-        i18n("&Export to Netscape Bookmarks"), "netscape", 0,
+        i18n("Export to &Netscape Bookmarks"), "netscape", 0,
         actn, SLOT( slotExportNS() ), actionCollection(), "exportNS");
     (void) new KAction(
-        i18n("&Export to Opera Bookmarks..."), "opera", 0,
+        i18n("Export to &Opera Bookmarks..."), "opera", 0,
         actn, SLOT( slotExportOpera() ), actionCollection(), "exportOpera");
     (void) new KAction(
-        i18n("&Export to HTML Bookmarks..."), "html", 0,
+        i18n("Export to &HTML Bookmarks..."), "html", 0,
         actn, SLOT( slotExportHTML() ), actionCollection(), "exportHTML");
     (void) new KAction(
-        i18n("&Export to IE Bookmarks..."), "ie", 0,
+        i18n("Export to &IE Bookmarks..."), 0,
         actn, SLOT( slotExportIE() ), actionCollection(), "exportIE");
     (void) new KAction(
         i18n("Export to &Mozilla Bookmarks..."), "mozilla", 0,
         actn, SLOT( slotExportMoz() ), actionCollection(), "exportMoz");
 }
 
-bool ActionsImpl::save() {
-    if (!CurrentMgr::self()->managerSave())
-        return false;
-    CurrentMgr::self()->notifyManagers();
-    KEBApp::self()->setModifiedFlag(false);
-    KEBApp::self()->m_cmdHistory->notifyDocSaved();
-    return true;
-}
-
-bool ActionsImpl::queryClose() {
-    if (!KEBApp::self()->m_modified)
-        return true;
-    if (KEBApp::self()->m_saveOnClose)
-        return save();
-
-    switch (
-            KMessageBox::warningYesNoCancel(
-                KEBApp::self(), i18n("The bookmarks have been modified.\n"
-                    "Save changes?"),
-                QString::null, KStdGuiItem::save(), KStdGuiItem::discard())
-           ) {
-        case KMessageBox::Yes:
-            return save();
-        case KMessageBox::No:
-            return true;
-        default: // case KMessageBox::Cancel:
-            return false;
-    }
-}
-
 void ActionsImpl::slotLoad() {
-    if (!queryClose())
-        return;
-    QString bookmarksFile 
+    QString bookmarksFile
         = KFileDialog::getOpenFileName(QString::null, "*.xml", KEBApp::self());
     if (bookmarksFile.isNull())
         return;
@@ -272,18 +222,17 @@ void ActionsImpl::slotLoad() {
     KEBApp::self()->construct();
 }
 
-void ActionsImpl::slotSave() {
-    (void)save();
-}
-
 void ActionsImpl::slotSaveAs() {
-    QString saveFilename 
+    KEBApp::self()->bkInfo()->commitChanges();
+    QString saveFilename
         = KFileDialog::getSaveFileName(QString::null, "*.xml", KEBApp::self());
     if (!saveFilename.isEmpty())
         CurrentMgr::self()->saveAs(saveFilename);
 }
 
 void CurrentMgr::doExport(ExportType type, const QString & _path) {
+    if(KEBApp::self())
+        KEBApp::self()->bkInfo()->commitChanges();
     QString path(_path);
     // TODO - add a factory and make all this use the base class
     if (type == OperaExport) {
@@ -295,7 +244,7 @@ void CurrentMgr::doExport(ExportType type, const QString & _path) {
 
     } else if (type == HTMLExport) {
         if (path.isNull())
-            path = KFileDialog::getSaveFileName( 
+            path = KFileDialog::getSaveFileName(
                         QDir::homeDirPath(),
                         i18n("*.html|HTML Bookmark Listing") );
         HTMLExporter exporter;
@@ -326,26 +275,31 @@ void KEBApp::setActionsEnabled(SelcAbilities sa) {
     KActionCollection * coll = actionCollection();
 
     QStringList toEnable;
-    
-    if (sa.itemSelected && !sa.root) {
+
+    if (sa.multiSelect || (sa.singleSelect && !sa.root))
         toEnable << "edit_copy";
-        if (!sa.urlIsEmpty && !sa.group && !sa.separator)
+
+    if (sa.multiSelect || (sa.singleSelect && !sa.root && !sa.urlIsEmpty && !sa.group && !sa.separator))
             toEnable << "openlink";
-    }
 
     if (!m_readOnly) {
         if (sa.notEmpty)
             toEnable << "testall" << "updateallfavicons";
 
-        if (sa.itemSelected) {
-            if (!sa.root)
+        if ( sa.multiSelect || (sa.singleSelect && !sa.root) )
                 toEnable << "delete" << "edit_cut";
+
+        if( sa.singleSelect)
             if (m_canPaste)
                 toEnable << "edit_paste";
-            if (!sa.separator)
-                toEnable << "testlink" << "updatefavicon";
-            toEnable << "showintoolbar";
-        }
+
+        if( sa.multiSelect || (sa.singleSelect && !sa.root && (sa.group || !sa.urlIsEmpty) && !sa.separator))
+            toEnable << "testlink" << "updatefavicon";
+
+        if(sa.multiSelect)
+            toEnable << "showintoolbar" << "hideintoolbar";
+        else if(sa.itemSelected)
+            toEnable << (sa.tbShowState ? "hideintoolbar" : "showintoolbar");
 
         if (sa.singleSelect && !sa.root && !sa.separator) {
             toEnable << "rename" << "changeicon" << "changecomment";
@@ -353,18 +307,14 @@ void KEBApp::setActionsEnabled(SelcAbilities sa) {
                 toEnable << "changeurl";
         }
 
-        if (!sa.multiSelect) {
+        if (sa.singleSelect) {
             toEnable << "newfolder" << "newbookmark" << "insertseparator";
             if (sa.group)
                 toEnable << "sort" << "recursivesort" << "setastoolbar";
         }
     }
 
-    QString stbString = QString("%1 in T&oolbar")
-        .arg(sa.tbShowState ? i18n("Hide") : i18n("Show"));
-    coll->action("showintoolbar")->setText(stbString);
-
-    for ( QStringList::Iterator it = toEnable.begin(); 
+    for ( QStringList::Iterator it = toEnable.begin();
             it != toEnable.end(); ++it )
     {
         coll->action((*it).ascii())->setEnabled(true);
@@ -381,27 +331,29 @@ void KEBApp::setCancelTestsEnabled(bool enabled) {
 }
 
 void ActionsImpl::slotCut() {
+    KEBApp::self()->bkInfo()->commitChanges();
     slotCopy();
-    KMacroCommand *mcmd 
-        = CmdGen::self()->deleteItems(i18n("Cut Items"), 
-                                      ListView::self()->selectedItems());
-    CmdHistory::self()->didCommand(mcmd);
+    DeleteManyCommand *mcmd = new DeleteManyCommand( i18n("Cut Items"), ListView::self()->selectedAddresses() );
+    CmdHistory::self()->addCommand(mcmd);
+
 }
 
 void ActionsImpl::slotCopy() {
+    KEBApp::self()->bkInfo()->commitChanges();
     // this is not a command, because it can't be undone
-    Q_ASSERT(ListView::self()->selectedItems()->count() != 0);
-    QValueList<KBookmark> bookmarks 
-        = ListView::self()->itemsToBookmarks(ListView::self()->selectedItems());
+    Q_ASSERT(ListView::self()->selectedItemsMap().count() != 0);
+    QValueList<KBookmark> bookmarks
+        = ListView::self()->itemsToBookmarks(ListView::self()->selectedItemsMap());
     KBookmarkDrag* data = KBookmarkDrag::newDrag(bookmarks, 0 /* not this ! */);
     kapp->clipboard()->setData(data, QClipboard::Clipboard);
 }
 
 void ActionsImpl::slotPaste() {
-    KMacroCommand *mcmd = 
-        CmdGen::self()->insertMimeSource(
-                            i18n("Paste"), 
-                            kapp->clipboard()->data(QClipboard::Clipboard), 
+    KEBApp::self()->bkInfo()->commitChanges();
+    KEBMacroCommand *mcmd =
+        CmdGen::insertMimeSource(
+                            i18n("Paste"),
+                            kapp->clipboard()->data(QClipboard::Clipboard),
                             ListView::self()->userAddress());
     CmdHistory::self()->didCommand(mcmd);
 }
@@ -409,6 +361,7 @@ void ActionsImpl::slotPaste() {
 /* -------------------------------------- */
 
 void ActionsImpl::slotNewFolder() {
+    KEBApp::self()->bkInfo()->commitChanges();
     bool ok;
     QString str = KInputDialog::getText( i18n( "Create New Bookmark Folder" ),
             i18n( "New folder:" ), QString::null, &ok );
@@ -422,6 +375,7 @@ void ActionsImpl::slotNewFolder() {
 }
 
 void ActionsImpl::slotNewBookmark() {
+    KEBApp::self()->bkInfo()->commitChanges();
     // TODO - make a setCurrentItem(Command *) which uses finaladdress interface
     CreateCommand * cmd = new CreateCommand(
                                 ListView::self()->userAddress(),
@@ -430,41 +384,48 @@ void ActionsImpl::slotNewBookmark() {
 }
 
 void ActionsImpl::slotInsertSeparator() {
+    KEBApp::self()->bkInfo()->commitChanges();
     CreateCommand * cmd = new CreateCommand(ListView::self()->userAddress());
     CmdHistory::self()->addCommand(cmd);
 }
 
-void ActionsImpl::slotImport() { 
-    // kdDebug() << "ActionsImpl::slotImport() where sender()->name() == " 
+void ActionsImpl::slotImport() {
+    KEBApp::self()->bkInfo()->commitChanges();
+    // kdDebug() << "ActionsImpl::slotImport() where sender()->name() == "
     //           << sender()->name() << endl;
-    ImportCommand* import 
+    ImportCommand* import
         = ImportCommand::performImport(sender()->name()+6, KEBApp::self());
     if (!import)
         return;
-    // TODO - following line doesn't work, fixme
-    ListView::self()->setInitialAddress(import->groupAddress());
     CmdHistory::self()->addCommand(import);
+    ListView::self()->setCurrent( ListView::self()->getItemAtAddress(import->groupAddress()), true);
 }
 
 // TODO - this is getting ugly and repetitive. cleanup!
 
 void ActionsImpl::slotExportOpera() {
+    KEBApp::self()->bkInfo()->commitChanges();
     CurrentMgr::self()->doExport(CurrentMgr::OperaExport); }
 void ActionsImpl::slotExportHTML() {
+    KEBApp::self()->bkInfo()->commitChanges();
     CurrentMgr::self()->doExport(CurrentMgr::HTMLExport); }
 void ActionsImpl::slotExportIE() {
+    KEBApp::self()->bkInfo()->commitChanges();
     CurrentMgr::self()->doExport(CurrentMgr::IEExport); }
 void ActionsImpl::slotExportNS() {
+    KEBApp::self()->bkInfo()->commitChanges();
     CurrentMgr::self()->doExport(CurrentMgr::NetscapeExport); }
 void ActionsImpl::slotExportMoz() {
+    KEBApp::self()->bkInfo()->commitChanges();
     CurrentMgr::self()->doExport(CurrentMgr::MozillaExport); }
-   
+
 /* -------------------------------------- */
 
 static QCString s_appId, s_objId;
 static KParts::ReadOnlyPart *s_part;
 
 void ActionsImpl::slotPrint() {
+    KEBApp::self()->bkInfo()->commitChanges();
     s_part = KParts::ComponentFactory
                         ::createPartInstanceFromQuery<KParts::ReadOnlyPart>(
                                 "text/html", QString::null);
@@ -480,30 +441,29 @@ void ActionsImpl::slotPrint() {
     KTempFile tmpf(locateLocal("tmp", "print_bookmarks"), ".html");
     QTextStream *tstream = tmpf.textStream();
     tstream->setEncoding(QTextStream::Unicode);
-    (*tstream) << exporter.toString(CurrentMgr::self()->mgr()->root());
+    (*tstream) << exporter.toString(CurrentMgr::self()->mgr()->root(), true);
     tmpf.close();
 
     s_appId = kapp->dcopClient()->appId();
     s_objId = s_part->property("dcopObjectId").toString().latin1();
     connect(s_part, SIGNAL(completed()), this, SLOT(slotDelayedPrint()));
 
-    s_part->openURL(tmpf.name());
+    s_part->openURL(KURL( tmpf.name() ));
 }
 
 void ActionsImpl::slotDelayedPrint() {
     Q_ASSERT(s_part);
-    DCOPRef(s_appId, s_objId).send("print", false); 
-    // delete s_part;  -- dies horribly atm
-    // TODO - is this a leak?
+    DCOPRef(s_appId, s_objId).send("print", false);
+    delete s_part;
     s_part = 0;
 }
 
 /* -------------------------------------- */
 
 void ActionsImpl::slotShowNS() {
+    KEBApp::self()->bkInfo()->commitChanges();
     bool shown = KEBApp::self()->nsShown();
     CurrentMgr::self()->mgr()->setShowNSBookmarks(shown);
-    KEBApp::self()->setModifiedFlag(true);
     // TODO - need to force a save here
     CurrentMgr::self()->reloadConfig();
 }
@@ -534,10 +494,12 @@ ActionsImpl::~ActionsImpl() {
 /* -------------------------------------- */
 
 void ActionsImpl::slotTestSelection() {
+    KEBApp::self()->bkInfo()->commitChanges();
     TestLinkItrHolder::self()->insertItr(new TestLinkItr(ListView::self()->selectedBookmarksExpanded()));
 }
 
 void ActionsImpl::slotUpdateFavIcon() {
+    KEBApp::self()->bkInfo()->commitChanges();
     FavIconsItrHolder::self()->insertItr(new FavIconsItr(ListView::self()->selectedBookmarksExpanded()));
 }
 
@@ -570,9 +532,10 @@ void KBookmarkGroupList::visitEnter(const KBookmarkGroup &grp) {
 }
 
 void ActionsImpl::slotRecursiveSort() {
-    KBookmark bk = ListView::self()->selectedItems()->first()->bookmark();
+    KEBApp::self()->bkInfo()->commitChanges();
+    KBookmark bk = ListView::self()->firstSelected()->bookmark();
     Q_ASSERT(bk.isGroup());
-    KMacroCommand *mcmd = new KMacroCommand(i18n("Recursive Sort"));
+    KEBMacroCommand *mcmd = new KEBMacroCommand(i18n("Recursive Sort"));
     KBookmarkGroupList lister(CurrentMgr::self()->mgr());
     QValueList<KBookmark> bookmarks = lister.getList(bk.toGroup());
     bookmarks << bk.toGroup();
@@ -585,7 +548,8 @@ void ActionsImpl::slotRecursiveSort() {
 }
 
 void ActionsImpl::slotSort() {
-    KBookmark bk = ListView::self()->selectedItems()->first()->bookmark();
+    KEBApp::self()->bkInfo()->commitChanges();
+    KBookmark bk = ListView::self()->firstSelected()->bookmark();
     Q_ASSERT(bk.isGroup());
     SortCommand *cmd = new SortCommand(i18n("Sort Alphabetically"), bk.address());
     CmdHistory::self()->addCommand(cmd);
@@ -594,12 +558,14 @@ void ActionsImpl::slotSort() {
 /* -------------------------------------- */
 
 void ActionsImpl::slotDelete() {
-    KMacroCommand *mcmd = CmdGen::self()->deleteItems(i18n("Delete Items"), ListView::self()->selectedItems());
-    CmdHistory::self()->didCommand(mcmd);
+    KEBApp::self()->bkInfo()->commitChanges();
+    DeleteManyCommand *mcmd = new DeleteManyCommand(i18n("Delete Items"), ListView::self()->selectedAddresses());
+    CmdHistory::self()->addCommand(mcmd);
 }
 
 void ActionsImpl::slotOpenLink() {
-    QValueList<KBookmark> bks = ListView::self()->itemsToBookmarks(ListView::self()->selectedItems());
+    KEBApp::self()->bkInfo()->commitChanges();
+    QValueList<KBookmark> bks = ListView::self()->itemsToBookmarks(ListView::self()->selectedItemsMap());
     QValueListIterator<KBookmark> it;
     for (it = bks.begin(); it != bks.end(); ++it) {
         if ((*it).isGroup() || (*it).isSeparator())
@@ -611,33 +577,45 @@ void ActionsImpl::slotOpenLink() {
 /* -------------------------------------- */
 
 void ActionsImpl::slotRename() {
+    KEBApp::self()->bkInfo()->commitChanges();
     ListView::self()->rename(KEBListView::NameColumn);
 }
 
 void ActionsImpl::slotChangeURL() {
+    KEBApp::self()->bkInfo()->commitChanges();
     ListView::self()->rename(KEBListView::UrlColumn);
 }
 
 void ActionsImpl::slotChangeComment() {
+    KEBApp::self()->bkInfo()->commitChanges();
     ListView::self()->rename(KEBListView::CommentColumn);
 }
 
 void ActionsImpl::slotSetAsToolbar() {
-    KBookmark bk = ListView::self()->selectedItems()->first()->bookmark();
+    KEBApp::self()->bkInfo()->commitChanges();
+    KBookmark bk = ListView::self()->firstSelected()->bookmark();
     Q_ASSERT(bk.isGroup());
-    KMacroCommand *mcmd = CmdGen::self()->setAsToolbar(bk);
+    KEBMacroCommand *mcmd = CmdGen::setAsToolbar(bk);
     CmdHistory::self()->addCommand(mcmd);
 }
 
 void ActionsImpl::slotShowInToolbar() {
-    KBookmark bk = ListView::self()->selectedItems()->first()->bookmark();
-    bool shown = CmdGen::self()->shownInToolbar(bk);
-    KMacroCommand *mcmd = CmdGen::self()->setShownInToolbar(bk, !shown);
+    KEBApp::self()->bkInfo()->commitChanges();
+    QValueList<KBookmark> bks = ListView::self()->itemsToBookmarks(ListView::self()->selectedItemsMap());
+    KEBMacroCommand *mcmd = CmdGen::setShownInToolbar(bks, true);
+    CmdHistory::self()->addCommand(mcmd);
+}
+
+void ActionsImpl::slotHideInToolbar() {
+    KEBApp::self()->bkInfo()->commitChanges();
+    QValueList<KBookmark> bks = ListView::self()->itemsToBookmarks(ListView::self()->selectedItemsMap());
+    KEBMacroCommand *mcmd = CmdGen::setShownInToolbar(bks, false);
     CmdHistory::self()->addCommand(mcmd);
 }
 
 void ActionsImpl::slotChangeIcon() {
-    KBookmark bk = ListView::self()->selectedItems()->first()->bookmark();
+    KEBApp::self()->bkInfo()->commitChanges();
+    KBookmark bk = ListView::self()->firstSelected()->bookmark();
     KIconDialog dlg(KEBApp::self());
     QString newIcon = dlg.selectIcon(KIcon::Small, KIcon::FileSystem);
     if (newIcon.isEmpty())
@@ -651,12 +629,10 @@ void ActionsImpl::slotChangeIcon() {
 
 void ActionsImpl::slotExpandAll() {
     ListView::self()->setOpen(true);
-    KEBApp::self()->setModifiedFlag(true);
 }
 
 void ActionsImpl::slotCollapseAll() {
     ListView::self()->setOpen(false);
-    KEBApp::self()->setModifiedFlag(true);
 }
 
 #include "actionsimpl.moc"
