@@ -1,4 +1,4 @@
-// -*- mode:cperl; cperl-indent-level:4; cperl-continued-statement-offset:4; indent-tabs-mode:nil -*-
+// -*- indent-tabs-mode:nil -*-
 // vim: set ts=4 sts=4 sw=4 et:
 /* This file is part of the KDE project
    Copyright (C) 2000 David Faure <faure@kde.org>
@@ -15,10 +15,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
+#include <config.h>
+
 #include "toplevel.h"
 #include "importers.h"
 
@@ -110,14 +112,14 @@ static int askUser(KApplication &app, QString filename, bool &readonly) {
 
 #include <kactioncollection.h>
 
-extern "C" int kdemain(int argc, char **argv) {
+extern "C" KDE_EXPORT int kdemain(int argc, char **argv) {
     KLocale::setMainCatalogue("konqueror");
-    KAboutData aboutData("keditbookmarks", I18N_NOOP("Bookmark Editor"), "1.2",
+    KAboutData aboutData("keditbookmarks", I18N_NOOP("Bookmark Editor"), VERSION,
             I18N_NOOP("Konqueror Bookmarks Editor"),
             KAboutData::License_GPL,
             I18N_NOOP("(c) 2000 - 2003, KDE developers") );
     aboutData.addAuthor("David Faure", I18N_NOOP("Initial author"), "faure@kde.org");
-    aboutData.addAuthor("Alexander Kellett", I18N_NOOP("Maintainer"), "lypanov@kde.org");
+    aboutData.addAuthor("Alexander Kellett", I18N_NOOP("Author"), "lypanov@kde.org");
 
     KCmdLineArgs::init(argc, argv, &aboutData);
     KApplication::addCmdLineOptions();
