@@ -771,8 +771,10 @@ void KSMServer::deleteClient( KSMClient* client )
     delete client;
     if ( state == Shutdown || state == Checkpoint )
         completeShutdownOrCheckpoint();
-    if ( state == KillingWM || state == Killing )
+    if ( state == Killing )
         completeKilling();
+    if ( state == KillingWM )
+        completeKillingWM();
 }
 
 void KSMServer::newConnection( int /*socket*/ )
@@ -900,13 +902,17 @@ QStringList KSMServer::sessionList()
 }
 
 bool KSMServer::isWM( const KSMClient* client ) const
+{
+    return isWM( client->program());
+}
+
+bool KSMServer::isWM( const QString& program ) const
 {
     // KWin relies on ksmserver's special treatment in phase1,
     // therefore make sure it's recognized even if ksmserver
     // was initially started with different WM, and kwin replaced
     // it later
-    return client->program() == wm
-        || client->program() == "kwin";
+    return program == wm || program == "kwin";
 }
 
 bool KSMServer::defaultSession() const
