@@ -54,6 +54,8 @@ public:
   ASYNC programStarted( QString programIcon, QString programName, QString description );
   ASYNC startupComplete();
   ASYNC close();
+  ASYNC hide();
+  ASYNC show();
 
 signals:
   void stepsChanged(int);
@@ -83,6 +85,7 @@ private slots:
 
 private:
   ThemeEngine *_loadThemeEngine( const QString& pluginName, const QString& theme );
+  void updateState( unsigned int state );
 
 protected:
   unsigned int mState;
