@@ -29,19 +29,10 @@
 
 #include <netwm.h>
 
-pid_t execute(const QCString & cmd){
-  KProcess proc;
-  proc.setUseShell(true);
-  proc << cmd;
-  if(proc.start(KProcess::DontCare))
-      return proc.pid();
-  return -1;
-}
-
 
 // some globals
 
-static QCString command = 0;
+static KProcess proc;
 static QCString windowtitle = 0;
 static QCString windowclass = 0;
 static int desktop = 0;
@@ -58,7 +49,7 @@ KStart::KStart()
     :QObject()
 {
     NETRootInfo i( qt_xdisplay(), NET::Supported );
-    bool useRule = i.isSupported( NET::WM2KDETemporaryRules );
+    bool useRule = !toSysTray && i.isSupported( NET::WM2KDETemporaryRules );
 
     if( useRule )
         sendRule();
@@ -74,16 +65,11 @@ KStart::KStart()
     id.setupStartupEnv();
 
     //finally execute the comand
-    pid_t pid = execute(command);
-
-    if( pid >= 0 ) {
+    if( proc.start(KProcess::DontCare) ) {
         KStartupInfoData data;
-        data.addPid( pid );
-        data.setName( command );
-        QCString bin = command;
-        int space = bin.find( ' ' ); // try to get the name of the binary
-        if( space != -1 )
-            bin = bin.left( space );
+        data.addPid( proc.pid() );
+        QCString bin = proc.args().first();
+        data.setName( bin );
         data.setBin( bin.mid( bin.findRev( '/' ) + 1 ));
         KStartupInfo::sendChange( id, data );
     }
@@ -271,7 +257,8 @@ static KCmdLineOptions options[] =
   { "window <regexp>", I18N_NOOP("A regular expression matching the window title"), 0 },
   { "windowclass <class>", I18N_NOOP("A string matching the window class (WM_CLASS property)\n"
                   "The window class can be found out by running\n"
-                  "'xprop | grep WM_CLASS' and clicking on a window.\n"
+                  "'xprop | grep WM_CLASS' and clicking on a window\n"
+                  "(use either both parts separated by a space or only the right part).\n"
                   "NOTE: If you specify neither window title nor window class,\n"
                   "then the very first window to appear will be taken;\n"
                   "omitting both options is NOT recommended."), 0 },
@@ -282,7 +269,7 @@ static KCmdLineOptions options[] =
   { "maximize", I18N_NOOP("Maximize the window"), 0 },
   { "maximize-vertically", I18N_NOOP("Maximize the window vertically"), 0 },
   { "maximize-horizontally", I18N_NOOP("Maximize the window horizontally"), 0 },
-  { "fullscreen", I18N_NOOP("Show window fullscreen. Implies type Override"), 0 },
+  { "fullscreen", I18N_NOOP("Show window fullscreen"), 0 },
   { "type <type>", I18N_NOOP("The window type: Normal, Desktop, Dock, Tool, \nMenu, Dialog, TopMenu or Override"), 0 },
   { "activate", I18N_NOOP("Jump to the window even if it is started on a \n"
                           "different virtual desktop"), 0 },
@@ -322,9 +309,8 @@ int main( int argc, char *argv[] )
   if ( args->count() == 0 )
       KCmdLineArgs::usage(i18n("No command specified"));
 
-  // Perhaps we should use a konsole-like solution here (shell, list of args...)
   for(int i=0; i < args->count(); i++)
-      command += QCString(args->arg(i)) + " ";
+    proc << args->arg(i);
 
   kwinmodule = new KWinModule;
 
