@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -22,10 +22,11 @@
 #include <kiconloader.h>
 #include <klocale.h>
 
-#include <ksgrd/ColorPicker.h>
 #include <ksgrd/SensorManager.h>
 #include <ksgrd/StyleEngine.h>
 
+#include <qfile.h>
+
 #include "SensorLogger.moc"
 #include "SensorLoggerSettings.h"
 
@@ -85,20 +86,18 @@ LogSensor::timerEvent(QTimerEvent*)
 void
 LogSensor::answerReceived(int id, const QString& answer)
 {
-	logFile = new QFile(fileName);
-	Q_CHECK_PTR(logFile);
+	QFile logFile(fileName);
 
-	if (!logFile->open(IO_ReadWrite | IO_Append))
+	if (!logFile.open(IO_ReadWrite | IO_Append))
 	{
 		stopLogging();
-		delete logFile;
 		return;
 	}
 
 	switch (id)
 	{
 		case 42: {
-			QTextStream stream(logFile);
+			QTextStream stream(&logFile);
 			double value = answer.toDouble();
 
 			if (lowerLimitActive && value < lowerLimit)
@@ -107,7 +106,7 @@ LogSensor::answerReceived(int id, const QString& answer)
 				lowerLimitActive = false;
 				lvi->setTextColor(monitor->colorGroup().foreground());
 				lvi->repaint();
-				KNotifyClient::event("sensor_alarm", QString("sensor '%1' at '%2' reached lower limit").arg(sensorName).arg(hostName));
+				KNotifyClient::event(monitor->winId(), "sensor_alarm", QString("sensor '%1' at '%2' reached lower limit").arg(sensorName).arg(hostName));
 				timerOn();
 			} else if (upperLimitActive && value > upperLimit)
 			{
@@ -115,7 +114,7 @@ LogSensor::answerReceived(int id, const QString& answer)
 				upperLimitActive = false;
 				lvi->setTextColor(monitor->colorGroup().foreground());
 				lvi->repaint();
-				KNotifyClient::event("sensor_alarm", QString("sensor '%1' at '%2' reached upper limit").arg(sensorName).arg(hostName));
+				KNotifyClient::event(monitor->winId(), "sensor_alarm", QString("sensor '%1' at '%2' reached upper limit").arg(sensorName).arg(hostName));
 				timerOn();
 			}
 			QDate date = QDateTime::currentDateTime().date();
@@ -125,8 +124,7 @@ LogSensor::answerReceived(int id, const QString& answer)
 		}
 	}
 
-	logFile->close();
-	delete logFile;
+	logFile.close();
 }
 
 SensorLogger::SensorLogger(QWidget *parent, const char *name, const QString& title)
