@@ -221,17 +221,18 @@ void Workspace::setActiveClient( Client* c, allowed_t )
         closeActivePopup();
     StackingUpdatesBlocker blocker( this );
     ++set_active_client_recursion;
+    updateFocusMousePosition( QCursor::pos());
     if( active_client != NULL )
         { // note that this may call setActiveClient( NULL ), therefore the recursion counter
-            active_client->setActive( false, !c || !c->isModal() || c != active_client->transientFor() );
-    }
+        active_client->setActive( false, !c || !c->isModal() || c != active_client->transientFor() );
+        }
     active_client = c;
     Q_ASSERT( c == NULL || c->isActive());
     if( active_client != NULL )
         last_active_client = active_client;
     if ( active_client ) 
         {
-        updateFocusChains( active_client, true ); // make it first in focus chain
+        updateFocusChains( active_client, FocusChainMakeFirst );
         active_client->demandAttention( false );
         }
     pending_take_activity = NULL;
@@ -265,6 +266,7 @@ void Workspace::activateClient( Client* c, bool force )
     {
     if( c == NULL )
         {
+        focusToNull();
         setActiveClient( NULL, Allowed );
         return;
         }
@@ -494,20 +496,20 @@ bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in
         return true;
     if( ac == NULL || ac->isDesktop())
         {
-        kdDebug( 1212 ) << "Activation: No client active, allowing" << endl;
+//        kdDebug( 1212 ) << "Activation: No client active, allowing" << endl;
         return true; // no active client -> always allow
         }
     // TODO window urgency  -> return true?
     if( Client::belongToSameApplication( c, ac, true ))
         {
-        kdDebug( 1212 ) << "Activation: Belongs to active application" << endl;
+//        kdDebug( 1212 ) << "Activation: Belongs to active application" << endl;
         return true;
         }
     if( level == 3 ) // high
         return false;
     if( time == -1U )  // no time known
         {
-        kdDebug( 1212 ) << "Activation: No timestamp at all" << endl;
+//        kdDebug( 1212 ) << "Activation: No timestamp at all" << endl;
         if( level == 1 ) // low
             return true;
         // no timestamp at all, don't activate - because there's also creation timestamp
@@ -517,8 +519,8 @@ bool Workspace::allowClientActivation( const Client* c, Time time, bool focus_in
         }
     // level == 2 // normal
     Time user_time = ac->userTime();
-    kdDebug( 1212 ) << "Activation, compared:" << c << ":" << time << ":" << user_time
-        << ":" << ( timestampCompare( time, user_time ) >= 0 ) << endl;
+//    kdDebug( 1212 ) << "Activation, compared:" << c << ":" << time << ":" << user_time
+//        << ":" << ( timestampCompare( time, user_time ) >= 0 ) << endl;
     return timestampCompare( time, user_time ) >= 0; // time >= user_time
     }
 
@@ -540,7 +542,7 @@ bool Workspace::allowFullClientRaising( const Client* c, Time time )
         return false;
     if( ac == NULL || ac->isDesktop())
         {
-        kdDebug( 1212 ) << "Raising: No client active, allowing" << endl;
+//        kdDebug( 1212 ) << "Raising: No client active, allowing" << endl;
         return true; // no active client -> always allow
         }
     if( c->ignoreFocusStealing())
@@ -548,14 +550,14 @@ bool Workspace::allowFullClientRaising( const Client* c, Time time )
     // TODO window urgency  -> return true?
     if( Client::belongToSameApplication( c, ac, true ))
         {
-        kdDebug( 1212 ) << "Raising: Belongs to active application" << endl;
+//        kdDebug( 1212 ) << "Raising: Belongs to active application" << endl;
         return true;
         }
     if( level == 3 ) // high
         return false;
     Time user_time = ac->userTime();
-    kdDebug( 1212 ) << "Raising, compared:" << time << ":" << user_time
-        << ":" << ( timestampCompare( time, user_time ) >= 0 ) << endl;
+//    kdDebug( 1212 ) << "Raising, compared:" << time << ":" << user_time
+//        << ":" << ( timestampCompare( time, user_time ) >= 0 ) << endl;
     return timestampCompare( time, user_time ) >= 0; // time >= user_time
     }
 
@@ -707,7 +709,7 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStar
     bool session ) const
     {
     Time time = info->userTime();
-    kdDebug( 1212 ) << "User timestamp, initial:" << time << endl;
+//    kdDebug( 1212 ) << "User timestamp, initial:" << time << endl;
     // newer ASN timestamp always replaces user timestamp, unless user timestamp is 0
     // helps e.g. with konqy reusing
     if( asn_data != NULL && time != 0 )
@@ -724,7 +726,7 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStar
             time = asn_data->timestamp();
             }
         }
-    kdDebug( 1212 ) << "User timestamp, ASN:" << time << endl;
+//    kdDebug( 1212 ) << "User timestamp, ASN:" << time << endl;
     if( time == -1U )
         { // The window doesn't have any timestamp.
       // If it's the first window for its application
@@ -756,7 +758,7 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStar
             // don't refuse if focus stealing prevention is turned off
             if( !first_window && rules()->checkFSP( options->focusStealingPreventionLevel ) > 0 )
                 {
-                kdDebug( 1212 ) << "User timestamp, already exists:" << 0 << endl;
+//                kdDebug( 1212 ) << "User timestamp, already exists:" << 0 << endl;
                 return 0; // refuse activation
                 }
             }
@@ -776,7 +778,7 @@ Time Client::readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStar
         else
             time = readUserCreationTime();
         }
-    kdDebug( 1212 ) << "User timestamp, final:" << this << ":" << time << endl;
+//    kdDebug( 1212 ) << "User timestamp, final:" << this << ":" << time << endl;
     return time;
     }
 
