@@ -75,6 +75,7 @@ Workspace::Workspace( bool restore )
     active_popup_client( NULL ),
     desktop_widget    (0),
     temporaryRulesMessages( "_KDE_NET_WM_TEMPORARY_RULES", NULL, false ),
+    rules_updates_disabled( false ),
     active_client     (0),
     last_active_client     (0),
     most_recently_raised (0),
@@ -289,6 +290,7 @@ void Workspace::init()
         NET::WM2ExtendedStrut |
         NET::WM2KDETemporaryRules |
         NET::WM2ShowingDesktop |
+        NET::WM2DesktopLayout |
         0
         ,
         NET::ActionMove |
@@ -309,6 +311,7 @@ void Workspace::init()
         protocols, 5, qt_xscreen() );
 
     loadDesktopSettings();
+    updateDesktopLayout();
     // extra NETRootInfo instance in Client mode is needed to get the values of the properties
     NETRootInfo client_info( qt_xdisplay(), NET::ActiveWindow | NET::CurrentDesktop );
     int initial_desktop;
@@ -396,7 +399,6 @@ void Workspace::init()
         NETSize desktop_geometry;
         desktop_geometry.width = geom.width();
         desktop_geometry.height = geom.height();
-    // TODO update also after gaining XRANDR support
         rootInfo->setDesktopGeometry( -1, desktop_geometry );
         setShowingDesktop( false );
 
@@ -572,6 +574,7 @@ void Workspace::removeClient( Client* c, allowed_t )
         focus_chain[ i ].remove( c );
     global_focus_chain.remove( c );
     attention_chain.remove( c );
+    showing_desktop_clients.remove( c );
     if( c->isTopMenu())
         removeTopMenu( c );
     Group* group = findGroup( c->window());
@@ -1257,15 +1260,16 @@ bool Workspace::setCurrentDesktop( int new_desktop )
     // and active_client is on_all_desktops and under mouse (hence == old_active_client),
     // conserve focus (thanks to Volker Schatz <V.Schatz at thphys.uni-heidelberg.de>)
     else if( active_client && active_client->isShown( true ) && active_client->isOnCurrentDesktop())
-      c= active_client;
+      c = active_client;
+
+    if( c == NULL && !desktops.isEmpty())
+        c = findDesktop( true, currentDesktop());
 
     if( c != active_client )
         setActiveClient( NULL, Allowed );
 
     if ( c ) 
         requestFocus( c );
-    else if( !desktops.isEmpty() ) 
-        requestFocus( findDesktop( true, currentDesktop()));
     else 
         focusToNull();
 
@@ -1519,25 +1523,33 @@ void Workspace::sendClientToDesktop( Client* c, int desk, bool dont_activate )
     updateClientArea();
     }
 
-void Workspace::setDesktopLayout(int o, int x, int y)
+void Workspace::setDesktopLayout( int, int, int )
+    { // DCOP-only, unused
+    }
+
+void Workspace::updateDesktopLayout()
     {
-    layoutOrientation = (Qt::Orientation) o;
-    layoutX = x;
-    layoutY = y;
+    // rootInfo->desktopLayoutCorner(); // I don't find this worth bothering, feel free to
+    layoutOrientation = ( rootInfo->desktopLayoutOrientation() == NET::OrientationHorizontal
+        ? Qt::Horizontal : Qt::Vertical );
+    layoutX = rootInfo->desktopLayoutColumnsRows().width();
+    layoutY = rootInfo->desktopLayoutColumnsRows().height();
+    if( layoutX == 0 && layoutY == 0 ) // not given, set default layout
+        layoutY = 2;
     }
 
 void Workspace::calcDesktopLayout(int &x, int &y) const
     {
-    x = layoutX;
+    x = layoutX; // <= 0 means compute it from the other and total number of desktops
     y = layoutY;
-    if ((x == -1) && (y > 0))
+    if((x <= 0) && (y > 0))
        x = (numberOfDesktops()+y-1) / y;
-    else if ((y == -1) && (x > 0))
+    else if((y <=0) && (x > 0))
        y = (numberOfDesktops()+x-1) / x;
 
-    if (x == -1)
+    if(x <=0)
        x = 1;
-    if (y == -1)
+    if (y <= 0)
        y = 1;
     }
 
