@@ -213,6 +213,8 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         WindowRules findWindowRules( const Client*, bool );
         void rulesUpdated();
         void discardUsedWindowRules( Client* c, bool withdraw );
+        void disableRulesUpdates( bool disable );
+        bool rulesUpdatesDisabled() const;
 
     // dcop interface
         void cascadeDesktop();
@@ -224,7 +226,8 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void circulateDesktopApplications();
 
         QString desktopName( int desk ) const;
-        void setDesktopLayout(int o, int x, int y);
+        virtual void setDesktopLayout(int , int , int );
+        void updateDesktopLayout();
         void setShowingDesktop( bool showing );
         void resetShowingDesktop( bool keep_hidden );
         bool showingDesktop() const;
@@ -279,6 +282,8 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         void cancelDelayFocus();
         void requestDelayFocus( Client* );
+        void updateFocusMousePosition( const QPoint& pos );
+        QPoint focusMousePosition() const;
         
         void toggleTopDockShadows(bool on);
 
@@ -386,7 +391,6 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
     protected:
         bool keyPressMouseEmulation( XKeyEvent& ev );
-        bool netCheck( XEvent* e );
 
     private:
         void init();
@@ -491,6 +495,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         QValueList<Rules*> rules;
         KXMessages temporaryRulesMessages;
         QTimer rulesUpdatedTimer;
+        bool rules_updates_disabled;
         static const char* windowTypeToTxt( NET::WindowType type );
         static NET::WindowType txtToWindowType( const char* txt );
         static bool sessionInfoWindowTypeMatch( Client* c, SessionInfo* info );
@@ -504,6 +509,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
     // delay(ed) window focus timer and client
         QTimer* delayFocusTimer;
         Client* delayfocus_client;
+        QPoint focusMousePos;
 
         ClientList clients;
         ClientList desktops;
@@ -774,6 +780,24 @@ inline bool Workspace::globalShortcutsDisabled() const
     return global_shortcuts_disabled || global_shortcuts_disabled_for_client;
     }
 
+inline
+bool Workspace::rulesUpdatesDisabled() const
+    {
+    return rules_updates_disabled;
+    }
+
+inline
+void Workspace::updateFocusMousePosition( const QPoint& pos )
+    {
+    focusMousePos = pos;
+    }
+
+inline
+QPoint Workspace::focusMousePosition() const
+    {
+    return focusMousePos;
+    }
+
 template< typename T >
 inline Client* Workspace::findClient( T predicate )
     {
