@@ -40,8 +40,9 @@ enum {
  LAYOUT_COLUMN_MAP = 2,
  LAYOUT_COLUMN_VARIANT = 3,
  LAYOUT_COLUMN_INCLUDE = 4,
+ LAYOUT_COLUMN_DISPLAY_NAME = 5,
  SRC_LAYOUT_COLUMN_COUNT = 3,
- DST_LAYOUT_COLUMN_COUNT = 5
+ DST_LAYOUT_COLUMN_COUNT = 6
 };
 
 static const QString DEFAULT_VARIANT_NAME("<default>");
@@ -117,6 +118,8 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
   connect( widget->listLayoutsDst, SIGNAL(selectionChanged(QListViewItem *)),
 		this, SLOT(layoutSelChanged(QListViewItem *)));
 
+  connect( widget->editDisplayName, SIGNAL(textChanged(const QString&)), this, SLOT(displayNameChanged(const QString&)));
+
   connect( widget->chkLatin, SIGNAL(clicked()), this, SLOT(changed()));
   connect( widget->chkLatin, SIGNAL(clicked()), this, SLOT(latinChanged()));
 
@@ -135,19 +138,23 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
   widget->listLayoutsSrc->setColumnText(LAYOUT_COLUMN_FLAG, "");
   widget->listLayoutsDst->setColumnText(LAYOUT_COLUMN_FLAG, "");
   widget->listLayoutsDst->setColumnText(LAYOUT_COLUMN_INCLUDE, "");
+//  widget->listLayoutsDst->setColumnText(LAYOUT_COLUMN_DISPLAY_NAME, "");
 
   widget->listLayoutsSrc->setColumnWidth(LAYOUT_COLUMN_FLAG, 28);
   widget->listLayoutsDst->setColumnWidth(LAYOUT_COLUMN_FLAG, 28);
 
   widget->listLayoutsDst->header()->setResizeEnabled(FALSE, LAYOUT_COLUMN_INCLUDE);
+  widget->listLayoutsDst->header()->setResizeEnabled(FALSE, LAYOUT_COLUMN_DISPLAY_NAME);
   widget->listLayoutsDst->setColumnWidthMode(LAYOUT_COLUMN_INCLUDE, QListView::Manual);
   widget->listLayoutsDst->setColumnWidth(LAYOUT_COLUMN_INCLUDE, 0);
+//  widget->listLayoutsDst->setColumnWidth(LAYOUT_COLUMN_DISPLAY_NAME, 0);
   
   widget->listLayoutsDst->setSorting(-1);
 #if 0
   widget->listLayoutsDst->setResizeMode(QListView::LastColumn);
   widget->listLayoutsSrc->setResizeMode(QListView::LastColumn);
 #endif
+  widget->listLayoutsDst->setResizeMode(QListView::LastColumn);
 
   //Read rules - we _must_ read _before_ creating xkb-options comboboxes
   loadRules();
@@ -179,6 +186,7 @@ void LayoutConfig::initUI() {
 	widget->comboModel->setCurrentText(i18n(modelName));
 
 	QValueList<LayoutUnit> otherLayouts = m_kxkbConfig.m_layouts;
+	widget->listLayoutsDst->clear();
 // to optimize we should have gone from it.end to it.begin
 	QValueList<LayoutUnit>::ConstIterator it;
 	for (it = otherLayouts.begin(); it != otherLayouts.end(); ++it ) {
@@ -193,10 +201,10 @@ void LayoutConfig::initUI() {
 				
 				newItem->setText(LAYOUT_COLUMN_VARIANT, layoutUnit.variant);
 				newItem->setText(LAYOUT_COLUMN_INCLUDE, layoutUnit.includeGroup);
+				newItem->setText(LAYOUT_COLUMN_DISPLAY_NAME, layoutUnit.displayName);
 				widget->listLayoutsDst->insertItem(newItem);
 				newItem->moveItem(widget->listLayoutsDst->lastItem());
 
-// 				m_includes[ layoutUnit ] = layoutUnit.includeGroup;
 				break;
 			}
 		}
@@ -274,13 +282,17 @@ void LayoutConfig::save()
 		QString layout = item->text(LAYOUT_COLUMN_MAP);
 		QString variant = item->text(LAYOUT_COLUMN_VARIANT);
 		QString includes = item->text(LAYOUT_COLUMN_INCLUDE);
+		QString displayName = item->text(LAYOUT_COLUMN_DISPLAY_NAME);
 		
 		LayoutUnit layoutUnit(layout, variant);
 		layoutUnit.includeGroup = includes;
+		layoutUnit.displayName = displayName;
 		layouts.append( layoutUnit );
 		
 		item = item->nextSibling();
-		kdDebug() << "Tosave: layout " << layoutUnit.toPair() << ", inc: " << layoutUnit.includeGroup << endl;
+		kdDebug() << "To save: layout " << layoutUnit.toPair() 
+				<< ", inc: " << layoutUnit.includeGroup 
+				<< ", disp: " << layoutUnit.displayName << endl;
 	}
 	m_kxkbConfig.m_layouts = layouts;
 
@@ -415,13 +427,42 @@ void LayoutConfig::variantChanged()
 }
 
 // helper
-static LayoutUnit getLayoutUnit(QListViewItem *sel)
+LayoutUnit LayoutConfig::getLayoutUnitKey(QListViewItem *sel)
 {
 	QString kbdLayout = sel->text(LAYOUT_COLUMN_MAP);
 	QString kbdVariant = sel->text(LAYOUT_COLUMN_VARIANT);
 	return LayoutUnit(kbdLayout, kbdVariant);
 }
 
+void LayoutConfig::displayNameChanged(const QString& newDisplayName)
+{
+	QListViewItem* selLayout = widget->listLayoutsDst->selectedItem();
+	if( selLayout == NULL )
+		return;
+	
+	const LayoutUnit layoutUnitKey = getLayoutUnitKey( selLayout );
+	LayoutUnit& layoutUnit = *m_kxkbConfig.m_layouts.find(layoutUnitKey);
+	
+	QString oldName = selLayout->text(LAYOUT_COLUMN_DISPLAY_NAME);
+	 
+	if( oldName.isEmpty() )
+		oldName = KxkbConfig::getDefaultDisplayName( layoutUnit );
+	
+	if( oldName != newDisplayName ) {
+		kdDebug() << "setting label for " << layoutUnit.toPair() << " : " << newDisplayName << endl;
+		selLayout->setText(LAYOUT_COLUMN_DISPLAY_NAME, newDisplayName);
+		updateIndicator(selLayout);
+		emit changed();
+	}
+}
+
+/** will update flag with label if layout label has been edited
+*/
+void LayoutConfig::updateIndicator(QListViewItem* selLayout)
+{
+}
+
+
 void LayoutConfig::latinChanged()
 {
     QListViewItem* selLayout = widget->listLayoutsDst->selectedItem();
@@ -438,24 +479,24 @@ void LayoutConfig::latinChanged()
 		include = "";
 	selLayout->setText(LAYOUT_COLUMN_INCLUDE, include);
 
- 	LayoutUnit layoutUnitKey = getLayoutUnit(selLayout);
+ 	LayoutUnit layoutUnitKey = getLayoutUnitKey(selLayout);
 	kdDebug() << "layout " << layoutUnitKey.toPair() << ", inc: " << include << endl;
 }
 
 void LayoutConfig::layoutSelChanged(QListViewItem *sel)
 {
     widget->comboVariant->clear();
-    widget->comboVariant->setEnabled( sel != 0 );
+    widget->comboVariant->setEnabled( sel != NULL );
     widget->chkLatin->setChecked( false );
-    widget->chkLatin->setEnabled( sel != 0 );
+    widget->chkLatin->setEnabled( sel != NULL );
 
-    if( sel == 0 ) {
+    if( sel == NULL ) {
         updateLayoutCommand();
         return;
     }
 
 
-	LayoutUnit layoutUnitKey = getLayoutUnit(sel);
+	LayoutUnit layoutUnitKey = getLayoutUnitKey(sel);
 	QString kbdLayout = layoutUnitKey.layout;
 
 	// TODO: need better algorithm here for determining if needs us group
@@ -547,10 +588,10 @@ QWidget* LayoutConfig::makeOptionsTab()
         QString text(it.current());
         text = text.replace( "Cap$", "Caps." );
         if( parent->type() == QCheckListItem::RadioButtonController )
-            new OptionListItem(parent, i18n(text.latin1()),
+            new OptionListItem(parent, i18n(text.utf8()),
                 QCheckListItem::RadioButton, key);
         else
-            new OptionListItem(parent, i18n(text.latin1()),
+            new OptionListItem(parent, i18n(text.utf8()),
                 QCheckListItem::CheckBox, key);
       }
     }
@@ -578,9 +619,10 @@ void LayoutConfig::updateOptionsCommand()
 void LayoutConfig::updateLayoutCommand()
 {
   QString setxkbmap;
+  QString layoutDisplayName;
   QListViewItem* sel = widget->listLayoutsDst->selectedItem();
 
-  if( sel ) {
+  if( sel != NULL ) {
     QString kbdLayout = sel->text(LAYOUT_COLUMN_MAP);
     QString variant = widget->comboVariant->currentText();
 	if( variant == DEFAULT_VARIANT_NAME )
@@ -589,10 +631,27 @@ void LayoutConfig::updateLayoutCommand()
     setxkbmap = "setxkbmap"; //-rules " + m_rule
     setxkbmap += " -model " + lookupLocalized(m_rules->models(), widget->comboModel->currentText())
       + " -layout ";
-    if( widget->chkLatin->isChecked() )
-      setxkbmap += "us,";
     setxkbmap += kbdLayout;
+    if( widget->chkLatin->isChecked() )
+      setxkbmap += ",us";
 
+/*	LayoutUnit layoutUnitKey = getLayoutUnitKey(sel);
+	layoutDisplayName = m_kxkbConfig.getLayoutDisplayName( *m_kxkbConfig.m_layouts.find(layoutUnitKey) );*/
+	layoutDisplayName = sel->text(LAYOUT_COLUMN_DISPLAY_NAME);
+	if( layoutDisplayName.isEmpty() ) {
+		int count = 0;
+		QListViewItem *item = widget->listLayoutsDst->firstChild();
+		while (item) {
+			QString layout_ = item->text(LAYOUT_COLUMN_MAP);
+			if( layout_ == kbdLayout )
+				++count;
+			item = item->nextSibling();
+		}
+		bool single = count < 2;
+		layoutDisplayName = m_kxkbConfig.getDefaultDisplayName(LayoutUnit(kbdLayout, variant), single);
+	}
+	kdDebug() << "disp: '" << layoutDisplayName << "'" << endl;
+	
     if( !variant.isEmpty() ) {
       setxkbmap += " -variant ";
       if( widget->chkLatin->isChecked() )
@@ -600,7 +659,11 @@ void LayoutConfig::updateLayoutCommand()
       setxkbmap += variant;
     }
   }
+  
   widget->editCmdLine->setText(setxkbmap);
+  
+  widget->editDisplayName->setEnabled( sel != NULL );
+  widget->editDisplayName->setText(layoutDisplayName);
 }
 
 void LayoutConfig::changed()
@@ -847,4 +910,67 @@ extern "C"
  I18N_NOOP("Right Alt key switches group while pressed");
  I18N_NOOP("Left Alt key switches group while pressed");
  I18N_NOOP("Press Right Alt-key to choose 3rd level");
+
+//new in Xorg 6.9
+ I18N_NOOP("R-Alt switches group while pressed.");
+ I18N_NOOP("Left Alt key switches group while pressed.");
+ I18N_NOOP("Left Win-key switches group while pressed.");
+ I18N_NOOP("Right Win-key switches group while pressed.");
+ I18N_NOOP("Both Win-keys switch group while pressed.");
+ I18N_NOOP("Right Ctrl key switches group while pressed.");
+ I18N_NOOP("Right Alt key changes group.");
+ I18N_NOOP("Left Alt key changes group.");
+ I18N_NOOP("CapsLock key changes group.");
+ I18N_NOOP("Shift+CapsLock changes group.");
+ I18N_NOOP("Both Shift keys together change group.");
+ I18N_NOOP("Both Alt keys together change group.");
+ I18N_NOOP("Both Ctrl keys together change group.");
+ I18N_NOOP("Ctrl+Shift changes group.");
+ I18N_NOOP("Alt+Ctrl changes group.");
+ I18N_NOOP("Alt+Shift changes group.");
+ I18N_NOOP("Menu key changes group.");
+ I18N_NOOP("Left Win-key changes group.");
+ I18N_NOOP("Right Win-key changes group.");
+ I18N_NOOP("Left Shift key changes group.");
+ I18N_NOOP("Right Shift key changes group.");
+ I18N_NOOP("Left Ctrl key changes group.");
+ I18N_NOOP("Right Ctrl key changes group.");
+ I18N_NOOP("Press Right Ctrl to choose 3rd level.");
+ I18N_NOOP("Press Menu key to choose 3rd level.");
+ I18N_NOOP("Press any of Win-keys to choose 3rd level.");
+ I18N_NOOP("Press Left Win-key to choose 3rd level.");
+ I18N_NOOP("Press Right Win-key to choose 3rd level.");
+ I18N_NOOP("Press any of Alt keys to choose 3rd level.");
+ I18N_NOOP("Press Left Alt key to choose 3rd level.");
+ I18N_NOOP("Press Right Alt key to choose 3rd level.");
+ I18N_NOOP("Ctrl key position");
+ I18N_NOOP("Make CapsLock an additional Ctrl.");
+ I18N_NOOP("Swap Ctrl and CapsLock.");
+ I18N_NOOP("Ctrl key at left of 'A'");
+ I18N_NOOP("Ctrl key at bottom left");
+ I18N_NOOP("Right Ctrl key works as Right Alt.");
+ I18N_NOOP("Use keyboard LED to show alternative group.");
+ I18N_NOOP("NumLock LED shows alternative group.");
+ I18N_NOOP("CapsLock LED shows alternative group.");
+ I18N_NOOP("ScrollLock LED shows alternative group.");
+ I18N_NOOP("CapsLock uses internal capitalization. Shift cancels CapsLock.");
+ I18N_NOOP("CapsLock uses internal capitalization. Shift doesn't cancel CapsLock.");
+ I18N_NOOP("CapsLock acts as Shift with locking. Shift cancels CapsLock.");
+ I18N_NOOP("CapsLock acts as Shift with locking. Shift doesn't cancel CapsLock.");
+ I18N_NOOP("CapsLock just locks the Shift modifier.");
+ I18N_NOOP("CapsLock toggles normal capitalization of alphabetic characters.");
+ I18N_NOOP("CapsLock toggles Shift so all keys are affected.");
+ I18N_NOOP("Alt and Meta are on the Alt keys (default).");
+ I18N_NOOP("Alt is mapped to the right Win-key and Super to Menu.");
+ I18N_NOOP("Compose key position");
+ I18N_NOOP("Right Alt is Compose.");
+ I18N_NOOP("Right Win-key is Compose.");
+ I18N_NOOP("Menu is Compose.");
+ I18N_NOOP("Right Ctrl is Compose.");
+ I18N_NOOP("Caps Lock is Compose.");
+ I18N_NOOP("Special keys (Ctrl+Alt+&lt;key&gt;) handled in a server.");
+ I18N_NOOP("Adding the EuroSign to certain keys");
+ I18N_NOOP("Add the EuroSign to the E key.");
+ I18N_NOOP("Add the EuroSign to the 5 key.");
+ I18N_NOOP("Add the EuroSign to the 2 key.");
 #endif
